C Program focalplot.f 
C Author: J. Pujol
C Plot of fault planes in a lower-hemisphere equal-area projection given the 
C parameters of one plane.
C To compile and link:
C f77 -c eigen2.f
C f77 -c nodplane-sub.f
C f77 -c pbt-sub.f
C ncargf77 -w focalplot.f nodplane-sub.o pbt-sub.o -o focalplot
C The output is in color. To view it on the screen enter:
C ctrans -d ps.color gmeta >out
C gv out
C The output can be sent to a black-and-white printer, as done for Fig. 10.11.
C The input file focal.data has the radius of the circle and adjustments for the 
C positions of several plot labels (described in the file).

	dimension xslp(3),yslp(3),xpbt(3),ypbt(3),chstr(3)
	common STRK(3),DDIP(3),SLIP(3),PTA(3),PTD(3),xtm(721,3),ytm(721,3)
     1,xx(721),yy(721),nf(2),mc1(2),mc2(2)	!,xtemp(1200),ytemp(1200)
     1,scra(1200),iscra(1200),xc(721),yc(721)
       dimension ias(13)  
        data ias /13*1/

	CHARACTER*1 PTCH(3),cnum*3,char*6,chstr*6
	DATA PTCH /'P','T','B'/

        CALL OPNGKS  	
	CALL PLCHHQ(.75,.995,'Output of focalplot.f',10.,0,0.)      
c	CALL PLCHHQ(.1,.97,'(a)',12.,0.,0.)	
c	CALL PLCHHQ(.1,.47,'(b)',12.,0.,0.)	
c        call plCHHQ(.75,.02,'Fig. 10-11a,b',10.,0,0.)

	open(unit=12,file='focal.data',status='old')
	iplot=0

999	PI4=ATAN(1.)
	RAD=PI4/45
c	PRINT*,' ENTER RADIUS OF CIRCLE (< 1):'
	READ(12,*)RC
	RC2=2*RC

6	PRINT*,' ENTER STRIKE, DIP AND SLIP OF PLANE:'
	READ(5,*)STRK(1),DDIP(1),SLIP(1)

	if(SLIP(1).lt.0)SLIP(1)=360+SLIP(1)

	call nodplane(STRK(1),DDIP(1),SLIP(1),STRK(2),DDIP(2),SLIP(2))
c	print*,STRK(1),DDIP(1),SLIP(1),STRK(2),DDIP(2),SLIP(2)
	call pbt(STRK(1),DDIP(1),SLIP(1),PTA(1),PTD(1),PTA(2),PTD(2),
     1PTA(3),PTD(3))
	
	STRK(3)=pta(3)+90	!strike and dip of plane perpendicular to
	DDIP(3)=90-ptd(3) 	!to the B axis; it contains the P and T axes
	SLIP(3)=999		!and the two slip vectors. SLIP(3) not used


	RCP=1.2*RC2
	RCP2=RCP/2

	CALL DASHDC('$''',5,5)
	call gsasf(IAS)
        call gsfais(1) 
        call dfclrs
        call sfseti('TYPE OF FILL',0)
	call gsplci(1)        

	YB=-.5
	YT=.5
	xl=-.5
	xr=.5

	WT=1
	WB=0.5
	WL=0.25
	WR=.75

	if(iplot.eq.1)then
	WT=0.5
	WB=0.
	end if

	LINEWIDTH=1500
	CALL SETUSV('LW',LINEWIDTH)
	ISYMBSIZ=100
	CALL SETUSV('MS',ISYMBSIZ)

	CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET

	CALL SET(WL,WR,WB,WT,XL,XR,YB,YT,1)  
	LINEWIDTH=2000
 	CALL SETUSV('LW',LINEWIDTH)

        CALL PLCHHQ(-.4,.43,':PRU::0725::B:1',10.,0.,0.)	!phi
        CALL PLCHHQ(-.4,.39,':PRU::0704::B:1',10.,0.,0.)	!delta
        CALL PLCHHQ(-.4,.35,':PRU::0713::B:1',10.,0.,0.)	!lambda
	write(cnum,'(i3)')nint(strk(1))
	char= ' = '//cnum
        CALL PLCHHQ(-.3,.43,char,10.,0.,0.)	
	write(cnum,'(i3)')nint(ddip(1))
	char= ' = '//cnum
        CALL PLCHHQ(-.3,.39,char,10.,0.,0.)	
	write(cnum,'(i3)')nint(slip(1))
	char= ' = '//cnum
        CALL PLCHHQ(-.3,.35,char,10.,0.,0.)	

        CALL PLCHHQ(.24,.43,':PRU::0725::B:2',10.,0.,0.)	!phi
        CALL PLCHHQ(.24,.39,':PRU::0704::B:2',10.,0.,0.)	!delta
        CALL PLCHHQ(.24,.35,':PRU::0713::B:2',10.,0.,0.)	!lambda
	write(cnum,'(i3)')nint(strk(2))
	char= ' = '//cnum
        CALL PLCHHQ(.34,.43,char,10.,0.,0.)	
	write(cnum,'(i3)')nint(ddip(2))
	char= ' = '//cnum
        CALL PLCHHQ(.34,.39,char,10.,0.,0.)	
	write(cnum,'(i3)')nint(slip(2))
	char= ' = '//cnum
        CALL PLCHHQ(.34,.35,char,10.,0.,0.)	

C PLOT CIRCLE
	RAD2=RAD/2
	DO I=1,721
c	ANG=(-I+361)*RAD2
	ANG=(i-1)*RAD2
	XC(I)=RC*COS(ANG)
	YC(I)=RC*SIN(ANG)
	xx(i)=xc(i)
	yy(i)=yc(i)
	END DO

C PLOT FOCAL PLANES
	SQR2=SQRT(2.)
	PI2=2*PI4
		nn=361
		DO K=1,3
	STRIKE=STRK(K)*RAD
	DIP=DDIP(K)*RAD
	SLP=SLIP(K)
	IF(SLP.GT.180)SLP=SLP-180
	ISL=0
	IF(SLP.GT.90)THEN
	SLP=180-SLP
	ISL=1
	END IF
	SLP=SLP*RAD
	DO I=1,nn
	AZIM=((I-1)/2.)*RAD
	APDIP=ATAN(TAN(DIP)*SIN(AZIM))
	RL=SQR2*RC*SIN(PI4-APDIP/2.)
	XC(I)=RL*SIN(AZIM+STRIKE)
	YC(I)=RL*COS(AZIM+STRIKE)
        xtm(i,k)=xc(i)
        ytm(i,k)=yc(i)
	END DO

c PLOT SLIP VECTORS
 	TANS=TAN(SLP)
	AZMS=ATAN(TANS*COS(DIP))
	IF(ISL.EQ.0)AZMS=180*RAD-AZMS
	SIND=SIN(DIP)
	SINS=SIN(SLP)
	SINDS2=SINS*SINS*SIND*SIND
		if(sinds2.eq.0)go to 6
	DIPS=(1-SINDS2)/SINDS2
	DIPS=SQRT(DIPS)
	DIPS=ATAN(DIPS)
	RL=SQR2*RC*SIN(DIPS/2.)
	X=RL*SIN(AZMS+STRIKE)
	Y=RL*COS(AZMS+STRIKE)
	xslp(k)=x
	yslp(k)=y
		END DO

C	PLOT P,T,B AXES
	DO I=1,3
	DIP=PTD(I)*RAD
	AZIM=PTA(I)*RAD
	RL=SQR2*RC*SIN(PI4-DIP/2.)
	X=RL*SIN(AZIM)
	Y=RL*COS(AZIM)
	xpbt(i)=x
	ypbt(i)=y
	if(i.eq.2)then
        xt=x  
        yt=y    
        end if
	if(i.eq.3)then
        xb=x  
        yb=y    
        end if
	END DO
	call shade(nn,xb,yb,xt,yt)
	call shade(nn,xb,yb,xt,yt)

	LINEWIDTH=2000
	CALL SETUSV('LW',LINEWIDTH)

        chstr(2)=':1245:'       !dot
        chstr(3)=':0541:'       !star
        chstr(1)=':1331:'       !circle
	do i=1,3	!P,B,T axes
	x=xpbt(i)
	y=ypbt(i)
	read(12,*)dx,dy
	call plCHHQ(x,y,chstr(i),20.,0,0.)
	call plCHHQ(x+dx,y+dy,ptch(i),20.,0,0.)
	end do

C	PLOT center
	DIP=90*RAD
	AZIM=dip
	RL=SQR2*RC*SIN(PI4-DIP/2.)
	Xcen=RL*SIN(AZIM)
	Ycen=RL*COS(AZIM)

	call plCHHQ(xcen,ycen,':1245:',10.,0,0.)
	call plCHHQ(xcen+.02,ycen+.02,'O',10.,0,0.)

	do k=1,2	!slip vectors and lines indicating dip
	k2=3-k
	x=xslp(k)
	y=yslp(k)
	call lined(x,y,xtm(nn/2,k2),ytm(nn/2,k2))

	call plCHHQ(x,y,':1246:',15.,0,0.)	!square
	ang=atan2(ytm(nn/2,k2)-y,xtm(nn/2,k2)-x)
	Xd=RC*COS(ANG)
	Yd=RC*SIN(ANG)
	ax=xtm(nn/2,k2)
	ay=ytm(nn/2,k2)
	call lined(xd,yd,ax,ay)
	call xyarr(xd,yd,ax,ay,.011,ang2,x1,y1)
        CALL PLCHHQ(x1,y1,':1213:',15.,ang2-90,0.)  !arrow
	call xyarr(ax,ay,xd,yd,.011,ang2,x1,y1)
        CALL PLCHHQ(x1,y1,':1213:',15.,ang2-90,0.)  !arrow
	if(k.eq.1)then
	read(12,*)dx,dy
        CALL PLCHHQ(.5*(ax+xd)+dx,.5*(ay+yd)+dy,':PRU::0704::B:2'
     1,10.,0.,0.)	!delta. The 2 is ok
	call xyarr(xcen,ycen,x,y,-.03,ang2,x1,y1)
	read(12,*)dx,dy
        CALL PLCHHQ(x1+dx,y1+dy,':PRU:s:B:1',10.,0.,0.)	!s
	else
	read(12,*)dx,dy
        CALL PLCHHQ(.5*(ax+xd)+dx,.5*(ay+yd)+dy,':PRU::0704::B:1'
     1,10.,0.,0.)	!delta. The 1 is ok
	read(12,*)dx,dy
	call xyarr(xcen,ycen,x,y,-.03,ang2,x1,y1)
        CALL PLCHHQ(x1+dx,y1+dy,':PRU:s:B:2',10.,0.,0.)	!s
	end if
	end do

	call curve(xx,yy,721)	!circle
	call line(rc,0,1.04*rc,0)
	call plCHHQ(1.1*rc,0,'E',20.,0,0.)
	call line(0,rc,0,1.04*rc)
	call plCHHQ(0,1.1*rc,'N',20.,0,0.)
	call line(0,-rc,0,-1.04*rc)
	call plCHHQ(0,-1.15*rc,'S',20.,0,0.)
	call line(-rc,0,-1.04*rc,0)
	call plCHHQ(-1.13*rc,0,'W',20.,0,0.)

		do i=1,3	!focal circles
	do j=1,nn
	xc(j)=xtm(j,i)
	yc(j)=ytm(j,i)
	end do

	if(i.eq.3)then
	call curved(xc,yc,nn)
	else
	call xyarr(xcen,ycen,xc(1),yc(1),-.03,ang2,x1,y1)
	if(i.eq.1)then
c	print*,' Enter dy for Pi1 and value to place it (integer < 360):'
	read(12,*)dy,nn2
        CALL PLCHHQ(xc(nn2),yc(nn2)+dy,':PRU::0720::B:1',10.,0.,0.)	!pi
        CALL PLCHHQ(x1,y1,':PRU::0725::B:1',10.,0.,0.)	!phi
	else
c	print*,' Enter dy for Pi2 and value to place it (integer < 360):'
	read(12,*)dy,nn2
        CALL PLCHHQ(xc(nn2),yc(nn2)+dy,':PRU::0720::B:2',10.,0.,0.)	!pi
        CALL PLCHHQ(x1,y1,':PRU::0725::B:2',10.,0.,0.)	!phi
	end if
	call curve(xc,yc,nn)
	ang=atan2(yc(2)-yc(1),xc(2)-xc(1))/rad
       CALL PLCHHQ(xc(2),yc(2),':1250:',25.,ANG,0.)
       CALL PLCHHQ(xc(2),yc(2),':1250:',15.,ANG,0.)
	end if
		end do

	iplot=iplot+1
	if(iplot.eq.2)go to 1000		
	print*,' ENTER 1 FOR ANOTHER PLOT:'
	read(5,*)iplot
	if(iplot.eq.1)go to 999

1000	call frame
	call clsgks
	STOP
	END

        subroutine dfclrs
C Define a set of RGB color triples for colors 1 through 16.
        dimension rgbv(3,16)   
	data rgbv /0.  ,0.  ,0.,  
     1             1  ,.75,.75,
     1             .7, .7, .7, 
     1             .75,.5, 1,  
     1             .5, 0,  1,  
     1             0,  0,  1,  
     1             0,  .5, 1,  
     1             0,  1,  1,  
     1             0,  1,  .6, 
     1             0,  1,  0,  
     1             .7, 1,  0,  
     1             1,  1,  0,  
     1             1,  .75,0,  
     1             1,  .38,.38,
     1             1,  0,  .38,
     1             1,  0,  0/  
 
        do i=1,16
        call gscr(1,i,rgbv(1,I),rgbv(2,I),rgbv(3,I))
        end do
        return
        end
C...................................................................................
C SHADE THE COMPRESSIONAL AREA

	subroutine shade(nn,xb,yb,xt,yt)
	common STRK(3),DDIP(3),SLIP(3),PTA(3),PTD(3),xtm(721,3),ytm(721,3)
     1,xx(721),yy(721),nf(2),mc1(2),mc2(2),xt2(1200),yt2(1200)
     1,scra(1200),iscra(1200),xc(721),yc(721)
	common /inside/ xtemp(1200),ytemp(1200)

	LINEWIDTH=1000
	CALL SETUSV('LW',LINEWIDTH)
C The intersection of the two focal circles is given by vector B
                do k=1,2
        do i=1,nn-1
        if(xtm(i,k).ge.xb.and.xtm(i+1,k).le.xb.or.
     1xtm(i,k).le.xb.and.xtm(i+1,k).ge.xb)then
        if(ytm(i,k).ge.yb.and.ytm(i+1,k).le.yb.or.
     1ytm(i,k).le.yb.and.ytm(i+1,k).ge.yb)then
	nf(k)=i
        go to 22
        end if
        end if
        end do
22              end do

		do k=1,2
		if(nf(k).eq.0)then
		print*,' k, nf(k) ....:', nf(k)
	dmin=9999
	do i=1,nn-1
	dist=(xb-xtm(i,k))**2+(yb-ytm(i,k))**2
	if(dist.lt.dmin)then
	xmin=xtm(i,k)
	ymin=ytm(i,k)
	dmin=dist
	imin=i
	end if
	end do
		nf(k)=imin
		end if
		end do

C Find index numbers for intersections of unit circle with focal circles
                do k=1,2
        dmin1=9999 
        dmin2=9999 
        do i=1,721 
        dist=sqrt((xx(i)-xtm(1,k))**2+(yy(i)-ytm(1,k))**2)
        if(dist.lt.dmin1)then
        mc1(k)=i
        dmin1=dist
        end if
        dist=sqrt((xx(i)-xtm(nn,k))**2+(yy(i)-ytm(nn,k))**2)
        if(dist.lt.dmin2)then
        mc2(k)=i
        dmin2=dist
        end if
        end do
                end do
 
C Build the closed contours
	k=0
	do i=1,nf(1)
	k=k+1
	xtemp(k)=xtm(i,1)
	ytemp(k)=ytm(i,1)
	xt2(k)=xtm(i,1)
	yt2(k)=ytm(i,1)
	end do
	do i=nf(2),nn
	k=k+1
	xtemp(k)=xtm(i,2)
	ytemp(k)=ytm(i,2)
	xt2(k)=xtm(i,2)
	yt2(k)=ytm(i,2)
	end do
               if(abs(mc2(2)-mc1(1)).gt.721/2)then
	do i=mc2(2),721
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	xt2(k)=xx(i)
	yt2(k)=yy(i)
	end do
	do i=1,mc1(1)
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	xt2(k)=xx(i)
	yt2(k)=yy(i)
	end do
		else
	deli=1
	if(mc2(2).gt.mc1(1))deli=-1
	do i=mc2(2),mc1(1),deli
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	xt2(k)=xx(i)
	yt2(k)=yy(i)
	end do
		end if

	kt2=k
	inside1=isinpo(xt,yt,k)

	k=0
	do i=1,nf(2)
	k=k+1
	xtemp(k)=xtm(i,2)
	ytemp(k)=ytm(i,2)
	end do
	do i=nf(1),nn
	k=k+1
	xtemp(k)=xtm(i,1)
	ytemp(k)=ytm(i,1)
	end do

	                if(abs(mc2(1)-mc1(2)).gt.721/2)then
		if(mc2(1).lt.mc1(2))then
	do i=mc2(1),1,-1
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
	do i=721,mc1(2),-1
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
		else
	do i=mc2(1),721
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
	do i=1,mc1(2)
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
		end if
			else
	deli=1
	if(mc2(1).gt.mc1(2))deli=-1
	do i=mc2(1),mc1(2),deli
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
			end if

	inside2=isinpo(xt,yt,k)
	if(inside1.ne.inside2)then
        call sfsgfa(xtemp,ytemp,k,scra,1200,iscra,1200,2)
        call sfsgfa(xt2,yt2,kt2,scra,1200,iscra,1200,2)
        call gsplci(1)
	return
	else
	go to 20
	end if
20	k=0
	do i=nf(1),nn
	k=k+1
	xtemp(k)=xtm(i,1)
	ytemp(k)=ytm(i,1)
	end do

			if(abs(mc2(1)-mc2(2)).gt.721/2)then
		if(mc2(1).gt.mc2(2))then
	do i=mc2(1),721
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
	do i=1,mc2(2)
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
		else
	do i=mc2(1),1,-1
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
	do i=721,mc2(2),-1
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
		end if

			else
	deli=1
	if(mc2(1).gt.mc2(2))deli=-1
	do i=mc2(1),mc2(2),deli
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
			end if
	do i=nn,nf(2),-1
	k=k+1
	xtemp(k)=xtm(i,2)
	ytemp(k)=ytm(i,2)
	end do

	inside=isinpo(xt,yt,k)
c	print*,' inside:',inside
        call sfsgfa(xtemp,ytemp,k,scra,1200,iscra,1200,2)
        call gsplci(1)

	k=0
	do i=nf(1),1,-1
	k=k+1
	xtemp(k)=xtm(i,1)
	ytemp(k)=ytm(i,1)
	end do

	                if(abs(mc1(1)-mc1(2)).gt.721/2)then
		if(mc1(1).lt.mc1(2))then
        do i=mc1(1),1,-1
        k=k+1
        xtemp(k)=xx(i)
        ytemp(k)=yy(i)
        end do
        do i=721,mc1(2),-1
        k=k+1
        xtemp(k)=xx(i)
        ytemp(k)=yy(i)
        end do
		else
        do i=mc1(1),721
        k=k+1
        xtemp(k)=xx(i)
        ytemp(k)=yy(i)
        end do
        do i=1,mc1(2)
        k=k+1
        xtemp(k)=xx(i)
        ytemp(k)=yy(i)
        end do
		end if
 	               else
	deli=1
	if(mc1(1).gt.mc1(2))deli=-1
	do i=mc1(1),mc1(2),deli
	k=k+1
	xtemp(k)=xx(i)
	ytemp(k)=yy(i)
	end do
			end if

	do i=1,nf(2)
	k=k+1
	xtemp(k)=xtm(i,2)
	ytemp(k)=ytm(i,2)
	end do

	inside=isinpo(xt,yt,k)
        call sfsgfa(xtemp,ytemp,k,scra,1200,iscra,1200,2)
        call gsplci(1)
	return
	end

      FUNCTION ISINPO (XPNT,YPNT,NP)

C  Here is such a code, based entirely on code written by Dave Kennison
C  here at NCAR
C   -Fred Clare

      PARAMETER (PI=3.14159265358979,TTPI=2.*PI)
C
      common /inside/ XP(1200),YP(1200)
C
C The value of this function is non-zero if and only if (XPNT,YPNT) is
C inside the polygon defined by the points (XP(I),YP(I)), for I from
C 1 to NP.
C
C Determine the number of points to look at (depending on whether the
C caller made the last point a duplicate of the first or not).
C
        IF (XP(NP).EQ.XP(1).AND.YP(NP).EQ.YP(1)) THEN
          NPTS=NP-1
        ELSE
          NPTS=NP
        END IF
C
C We compute the total angular change described by a ray emanating from
C the point (XPNT,YPNT) and passing through a point that moves around
C the polygon.
C
        ANCH=0.
C
        INXT=NPTS
        XNXT=XP(NPTS)
        YNXT=YP(NPTS)
        IF (XNXT.EQ.XPNT.AND.YNXT.EQ.YPNT) THEN
          ISINPO=0
          RETURN
        ELSE
          ANXT=ATAN2(YNXT-YPNT,XNXT-XPNT)
        END IF
C
        DO 101 IPNT=1,NPTS
          ILST=INXT
          XLST=XNXT
          YLST=YNXT
          ALST=ANXT
          INXT=IPNT
          XNXT=XP(INXT)
          YNXT=YP(INXT)
          IF (XNXT.EQ.XPNT.AND.YNXT.EQ.YPNT) THEN
            ISINPO=0
            RETURN
          ELSE
            ANXT=ATAN2(YNXT-YPNT,XNXT-XPNT)
          END IF
          ADIF=ANXT-ALST
          IF (ABS(ADIF).GT.PI) ADIF=ADIF-SIGN(TTPI,ADIF)
          ANCH=ANCH+ADIF
  101   CONTINUE
C
C If the point is outside the polygon, the total angular change should
C be exactly zero, while if the point is inside the polygon, the total
C angular change should be exactly plus or minus two pi.  We just test
C for the absolute value of the change being less than or equal to pi.
C
        IF (ABS(ANCH).LT.PI) THEN
          ISINPO=0
        ELSE
          ISINPO=1
        END IF
C
C Done.
C
        RETURN
C
      END
c------------------------------------------------------------------------------
        subroutine xyarr(x1,y1,x2,y2,dr,ang,xa,ya)
	pi=4*atan(1.)
	radpi=pi/180
        ang=atan2(y2-y1,x2-x1)/radpi
        r=sqrt((y2-y1)**2+(x2-x1)**2)
        r2=r-dr
        xa=x1+r2*cos(radpi*ang)
        ya=y1+r2*sin(radpi*ang)
        return
        end




