C Program hilbert.f
C Author: J. Pujol
C Displacements in the time domain for incident SV waves (free surface) for angles 
C larger than the critical. Uses the function given in Cerveny et al., Ray method in 
C seismology, 1977, p.49 (stored in the array y).
C To compile and link:
C f77 -c fftsub.f
C ncargf77 -w hilbert.f fftsub.o -o hilbert

	dimension t(512),yo(512),t2(512),a1(512),b1(512),f(6)
     1,xtmp(512),ytmp(512)
	COMMON nn,x(512),y(512),amp(512),com(512)
        COMMON /cchi/sinchi,coschi,sin2chi,cos2chi,fcr,tanf,tan2f
	data f /0.,40.,45.,50.,55.,60./
	complex com
	character*12 char

	const=12-13./14.
	pi=4*atan(1.)
	rad=pi/180.
	fre=1
	gamma=3
	fac1=-(2*pi*fre/gamma)**2
	fac2=2*pi*fre
	nn=128
	nn2=nn/2
	n3=100
	del=.03
	phase=70*rad
	do i=1,nn
	t(i)=(i-nn2)*del
	t2(i)=(n3+i)*del
	y(i)=exp(fac1*t(i)**2)*cos(fac2*t(i)+phase)
	end do

	tr=.0048
	do i=1,nn
	if(abs(y(i)).gt.tr)then
	t0=t(i)
	t02=t2(i)
	go to 10
	else
	y(i)=0
	end if
	end do

10	do i=1,nn
	yo(i)=y(i)+const	!const added for plotting
	end do

	XL=t(1)
        XR=t2(nn)
        YB=0
        YB=-1
        YT=12
	WT=1
        WB=.1
        WL=0
        WR=.35
        CALL OPNGKS
c        CALL PLCHHQ(.5*(wl+wr),.05,'Fig. 6-5',10.,0.,0.)
	CALL PLCHHQ(.5,.965,'Output of hilbert.f',10.,0,0.)
        CALL SET(WL,WR,WB,WT,XL,XR,YB,YT,1)
        CALL SETUSV('LW',1500)

	call curve(t,yo,nn)
        CALL PLCHHQ(t0,const-.1,':734:',12.,0.,0.)
        CALL PLCHHQ(t0,const-.27,'t=0',10.,0.,0.)
        CALL PLCHHQ(t(2),const+.6,'b:S1:h (t)',10.,0.,-1.)	

	do i=1,nn
	yo(i)=yo(i)-const
	end do

C COMPUTE HILBERT TRANSFORM USING FFT
	DO I=1,nn
	COM(I)=CMPLX(y(I),0.)
	END DO

C The following inefficient way of doing this is due to the fact that a copyrighted
C FFT subroutine was replaced by one in the public domain with different arguments
	do i=1,nn
	xtmp(i)=real(com(i))
	ytmp(i)=aimag(com(i))
	end do
	call fft(xtmp,ytmp,nn,nn,nn,-1)
	do i=1,nn
	com(i)=cmplx(xtmp(i),ytmp(i))
	end do

	CALL HILBERT
	FAC=1./nn	
	DO I=1,nn
	x(I)=FAC*REAL(COM(I))+const
	y(I)=FAC*AIMAG(COM(I))+const
	END DO
	call curve(t2,x,nn)
c	call curve(t2,y,nn)
        CALL PLCHHQ(t02,const-.1,':734:',12.,0.,0.)	!arrow
        CALL PLCHHQ(t02,const-.27,'t=0',10.,0.,0.)
        CALL PLCHHQ(t2(2),const+.6,'b:S1:h (t)',10.,0.,-1.)	
c        CALL PLCHHQ(t2(4),const+.77,':1330:',12.,0.,-1.)	!hat
        CALL PLCHHQ(t2(4),const+.77,':1313:',12.,0.,-1.)	!
	do i=1,nn
	x(i)=x(i)-const
	end do

C Plot pulses in the time domain for f > or = fc
c	call subchi(1.)	!!!! this gives access violation error
C For f=45 degrees, chi=0, sinchi=0 and tan2f goes to infinity, but the
C computer gives the expected value of 0 for a1
	f(1)=37	!the amplitude for f=fc is too large

        CALL PLCHHQ(t(2),const+.65-(13./7.),'a:B1:1:S1:h (t)'
     1,10.,0.,-1.)	
        CALL PLCHHQ(t2(2),const+.65-(13./7.),'b:B1:1:S1:h (t)'
     1,10.,0.,-1.)	
        CALL PLCHHQ(t0-.1,const-.27-(13./7.),'t=t:B2:P1',10.,0.,0.)
        CALL PLCHHQ(t02-.1,const-.27-(13./7.),'t=t:B2:S1',10.,0.,0.)
	th=.5*(xl+xr)

	char(4:12)=':S1::632:'
		do j=1,6
		const2=const-j*13./7.
	call subchi(f(j))
	fac=2*tanf*tan2f*sinchi
	do i=1,nn
	a1(i)=fac*(sinchi*yo(i)+coschi*x(i))+const2	!a1(t)
	b1(i)=-cos2chi*yo(i)+sin2chi*x(i)+const2		!b1(t)
	end do
	call curve(t,a1,nn)
        CALL PLCHHQ(t0,const2-.1,':734:',12.,0.,0.)	!arrow
	call curve(t2,b1,nn)
        CALL PLCHHQ(t02,const2-.1,':734:',12.,0.,0.)	!arrow
	ia=f(j)+.1
	write(char(2:3),'(i2)')ia
        CALL PLCHHQ(th,const2+.1,char(2:12),10.,0.,0.)

		end do

900     call frame
        call clsgks
        stop
        end	
C-------------------------------------------------------------------------------
	SUBROUTINE HILBERT
C TO COMPUTE THE HILBERT TRANSFORM MULTIPLY BY I*SGN(FREQ) AND THEN COMPUTE THE
C INVERSE FFT
C
C DEFINITION OF THE SIGN FUNCTION FOR DISCRTE SEQUENCES:
C =  0 for the zero and Nyquist frequencies, 
C =  1 for the positive frequencies
C = -1 for the negative frequencies
C 
C (This is for N even, see Bose, Digital Filters, eqn. 2.35a)
C
C For the FFT, with frequencies n/DT, n=1,2,...,N, the definition given above
C corresponds to the following indices: 
C       1 and N/2 + 1
C       2, 3, ..., N/2
C 	N/2 + 2, ...,  N,
C respectively.

	dimension xtmp(512),ytmp(512)
	COMMON N,XR(512),XI(512),AMP(512),COM(512)
	COMPLEX COM

	DO I=1,N
	XR(I)=REAL(COM(I))
	XI(I)=AIMAG(COM(I))
	END DO

	NN2=N/2
	DO I=2,NN2
	COM(I)=CMPLX(-XI(I),XR(I))
	END DO
	DO I=NN2+2,N
	COM(I)=CMPLX(XI(I),-XR(I))
	END DO
	COM(NN2+1)=CMPLX(0.,0.)
	COM(1)=CMPLX(0.,0.)

	DO I=1,N
	COM(I)=CONJG(COM(I))
	END DO

	do i=1,n
	xtmp(i)=real(com(i))
	ytmp(i)=aimag(com(i))
	end do
	call fft(xtmp,ytmp,n,n,n,-1)
	do i=1,n
	com(i)=cmplx(xtmp(i),ytmp(i))
	end do

	DO I=1,N
	COM(I)=CONJG(COM(I))
	END DO

	RETURN
	END
C-------------------------------------------------------------------------------
	subroutine subchi(f)
        COMMON /cchi/sinchi,coschi,sin2chi,cos2chi,fcr,tanf,tan2f

	vpvs=1.732
	alpha=4
	beta=alpha/vpvs
	albe=alpha/beta
	albe2=albe**2
	beal=beta/alpha
	beal2=beal**2
C fc = critical angle
	fc=asin(beal)
	pi=4*atan(1.)
	rad=pi/180.
	radinv=1./rad
	fcr=fc/rad

	f=f*rad
	tanf=tan(f)
	tan2f=tan(2*f)
		if(f.lt.fc)then
	print*,' Angle less than critical'
	return
		elseif(f.eq.fc)then
	print*,' Angle equal to  critical'
	chi=90*rad	
		elseif(f.gt.fc)then
	sinf=sin(f)
	sinf2=sinf**2
	cos2f=cos(2*f)
	cosf=cos(f)
	cot2f=1./tan2f
	chi=atan((cos2f*cot2f)/(2*sinf*sqrt(sinf2-beal2)))
		end if

	sinchi=sin(chi)
	coschi=cos(chi)
	sin2chi=sin(2*chi)
	cos2chi=cos(2*chi)
	f=f*radinv
c	print*,f
	RETURN
	END
