C Program p-sv-layer.f
C Author: J. Pujol

C Solid layer over solid half-space. 

C Incident P or SV wave

C alpha1, alpha: P wave velocity in the layer and half-space
C beta1, beta: S wave velocity in the layer and half-space
C rho1,rho: density in the layer and half-space
C h: layer thickness
C f: incidence angle at the bottom of layer
C u0: horizontal component of surface displacement
C w0: vertical component of surface displacement
C wk: wave number k = omega/c

C The subroutine used to solve the 4x4 system of equations was part of a
C commercial package and has been replaced here by a much simpler one
C (see subroutine equation). The new subroutine seems to perform well, 
C but phases of pi may become -pi or viceversa (which is not a problem).

C Input file: layer2.model

        common /amppha/ Yhc(5000),Yvc(5000),XX(5000),uphase(5000),
     1wphase(5000)
	common /param/ h,alpha1,beta1,rho1,alpha,beta,rho,ips,isurf,nper
     1,fdeg,edeg
	common /splot/ wl,wr,wb,wt,xxl,xxr
	complex one,ci,etaal1,etabe1,etaal,etabe,amtx(6,7),xvec(6),yv(6)
     1,wkh,u0,w0,uh,wh
	complex ual1,ube1,wal1,wbe1,tal1,tbe1,sal1,sbe1,eual1,eube1,
     1edal1,edbe1
	complex ual,ube,wal,wbe,tal,tbe,sal,sbe,eual,eube,
     1edal,edbe
	COMMON /plot/ XL,XR,YB,YT,FC,NN,IC,px,ppy,delinv

	character*9 char,char2*60,charf*9

	e=0
	f=0
	ci=cmplx(0.,1.)
	one=cmplx(1.,0.)
	pi=4*atan(1.)
	twopi=2*pi
	rad=pi/180
	radinv=1./rad

        alpha1=6.285
        beta1=3.635
        Alpha=7.96
        Beta=4.6
        rho1=2.869
        rho=3.37
        h=37

	print*,' Enter H, and beta and rho of layer and half-space:'
	open(unit=1,file='layer2.model',status='old')
	read(1,*)h,alpha1,beta1,rho1,alpha,beta,rho

	alphasq=alpha**2
	alpha1sq=alpha1**2
	betasq=beta**2
	beta1sq=beta1**2

	print*,' ENTER 1 for P waves; 0 for S waves and min. and max. period:'
	read(5,*)ips,xl,xr
        xxl=xl
        xxr=xr
	print*,' Enter 1 for surface displacement and 0 for bottom 
     1displacement:'
	read(5,*)isurf

	if(ips.eq.0)then
	print*,' Enter angle f (S waves):'
	read(5,*)fdeg
	f=rad*fdeg
	sinf=sin(f)
	sine=(alpha/beta)*sinf
	sinf1=(beta1/beta)*sinf
	sine1=(alpha1/beta)*sinf
	c=beta/sinf
	else
	print*,' Enter angle e (P waves):'
	read(5,*)edeg
	e=rad*edeg
	sine=sin(e)
	sinf=(beta/alpha)*sine
	sinf1=(beta1/alpha)*sine
	sine1=(alpha1/alpha)*sine
	c=alpha/sine
	end if

	csq=c*c
	gam1=2*beta1sq/csq
	gam=2*betasq/csq
	alc=(alpha/c)**2	
	alc1=(alpha1/c)**2	
	bec=(beta/c)**2	
	bec1=(beta1/c)**2	
C eta_beta for layer
	fac=csq/beta1sq
	if(fac.ge.1)then
	etabe1=cmplx(sqrt(fac-1),0.)	
	else
	etabe1=cmplx(0.,-sqrt(1-fac))	
	end if
C eta_beta for half-space
	fac=csq/betasq
	if(fac.ge.1)then
	etabe=cmplx(sqrt(fac-1),0.)	
	else
	etabe=cmplx(0.,-sqrt(1-fac))	
	end if

C eta alpha for layer
	fac=csq/alpha1sq
	if(fac.ge.1)then
	etaal1=cmplx(sqrt(fac-1),0.)	
	else
	etaal1=cmplx(0.,-sqrt(1-fac))	
	end if
C eta alpha for half-space
	fac=csq/alphasq
	if(fac.ge.1)then
	etaal=cmplx(sqrt(fac-1),0.)	
	else
	etaal=cmplx(0.,-sqrt(1-fac))	
	end if

1	nper=5000
        dper=(xr-xl)/nper

		        DO II=1,NPER
        t=xl+(ii-1)*dper
        XX(II)=ALOG10(T)
	freq=1./t
	omega=twopi*freq
	wk=omega/c
	wkh=wk*h*ci
	wkal1=one*(wk/sine1)
	wkbe1=one*(wk/sinf1)
	wkal=one*(wk/sine)
	wkbe=one*(wk/sinf)

	ual1=one*(wkal1*alc1)
	ube1=etabe1*wkbe1*bec1
	wal1=etaal1*wkal1*alc1
	wbe1=one*(wkbe1*bec1)
	tal1=rho1*gam1*wkal1*alpha1sq*etaal1
	tbe1=-.5*rho1*csq*wkbe1*gam1*(gam1-1)
	sal1=one*(wkal1*alpha1sq*rho1*(gam1-1))
	sbe1=.5*rho1*wkbe1*gam1**2*csq*etabe1
	eual1=cexp(wkh*etaal1)
	eube1=cexp(wkh*etabe1)
	edal1=cexp(-wkh*etaal1)
	edbe1=cexp(-wkh*etabe1)

	ual=one*(wkal*alc)
	ube=etabe*wkbe*bec
	wal=etaal*wkal*alc
	wbe=one*(wkbe*bec)
	tal=rho*gam*wkal*alphasq*etaal
	tbe=-.5*rho*csq*wkbe*gam*(gam-1)
	sal=one*(wkal*alphasq*rho*(gam-1))
	sbe=.5*rho*wkbe*gam**2*csq*etabe
	eual=cexp(wkh*etaal)
	eube=cexp(wkh*etabe)
	edal=cexp(-wkh*etaal)
	edbe=cexp(-wkh*etabe)

C We actually solve for the product of the amplitude factors time the 
C exponential. To get the amplitude factors the elements of the solution 
C vectors are divided by the corresponding exponentials. 

        amtx(1,1)=tal1*edal1
        amtx(1,2)=-tal1*eual1
        amtx(1,3)=tbe1*edbe1
        amtx(1,4)=tbe1*eube1
        amtx(1,5)=0*one
        amtx(1,6)=0*one
        amtx(2,1)=sal1*edal1
        amtx(2,2)=sal1*eual1
        amtx(2,3)=sbe1*edbe1
        amtx(2,4)=-sbe1*eube1
        amtx(2,5)=0*one
        amtx(2,6)=0*one
        amtx(3,1)=ual1
        amtx(3,2)=ual1
        amtx(3,3)=ube1
        amtx(3,4)=-ube1
        amtx(3,5)=-ual
        amtx(3,6)=ube
        amtx(4,1)=-wal1
        amtx(4,2)=wal1
        amtx(4,3)=wbe1
        amtx(4,4)=wbe1
        amtx(4,5)=-wal
        amtx(4,6)=-wbe
        amtx(5,1)=tal1
        amtx(5,2)=-tal1
        amtx(5,3)=tbe1
        amtx(5,4)=tbe1
        amtx(5,5)=tal
        amtx(5,6)=-tbe
        amtx(6,1)=sal1
        amtx(6,2)=sal1
        amtx(6,3)=sbe1
        amtx(6,4)=-sbe1
        amtx(6,5)=-sal
        amtx(6,6)=sbe

	if(ips.eq.0)then
        xvec(1)=0*one
        xvec(2)=0*one
        xvec(3)=ube*eube
        xvec(4)=wbe*eube
        xvec(5)=tbe*eube
        xvec(6)=sbe*eube
	else
	xvec(1)=0*one
        xvec(2)=0*one
        xvec(3)=ual*eual
        xvec(4)=-wal*eual
        xvec(5)=tal*eual
        xvec(6)=sal*eual
	end if

	do ik=1,6
	amtx(ik,7)=xvec(ik)
	end do
	call equation(amtx,yv,6)

C yv(1)=A'u;   yv(2)=A'd;   yv(3)=B'u;   yv(4)=B'd	(layer)
C yv(5)=Ad;    yv(6)=Bd					(half-space)
        yv(1)=yv(1)/eual1
        yv(2)=yv(2)/edal1
        yv(3)=yv(3)/eube1
        yv(4)=yv(4)/edbe1
        yv(5)=yv(5)/edal
        yv(6)=yv(6)/edbe

	if(isurf.eq.1)then
C Surface displacement
	u0=(1./wk)*((yv(1)+yv(2))*ual1+(yv(3)-yv(4))*ube1)
	w0=(1./wk)*((-yv(1)+yv(2))*wal1+(yv(3)+yv(4))*wbe1)
	elseif(isurf.eq.0)then
C Displacement at the bottom of the layer
	u0=(1./wk)*((yv(1)*eual1+yv(2)*edal1)*ual1
     1+(yv(3)*eube1-yv(4)*edbe1)*ube1)
	w0=(1./wk)*((-yv(1)*eual1+yv(2)*edal1)*wal1
     1+(yv(3)*eub1+yv(4)*edbe1)*wbe1)
	elseif(isurf.eq.-1)then	!ratio of surface and bottom displacements
C This option is not used because the plot not always comes right
	u0=(1./wk)*((yv(1)+yv(2))*ual1+(yv(3)-yv(4))*ube1)
	uh=(1./wk)*((yv(1)*eual1+yv(2)*edal1)*ual1
     1+(yv(3)*eube1-yv(4)*edbe1)*ube1)
	u0=u0/uh
	w0=(1./wk)*((-yv(1)+yv(2))*wal1+(yv(3)+yv(4))*wbe1)
	wh=(1./wk)*((-yv(1)*eual1+yv(2)*edal1)*wal1
     1+(yv(3)*eub1+yv(4)*edbe1)*wbe1)
	w0=w0/wh
	end if

	yhc(ii)=cabs(u0)
	yvc(ii)=cabs(w0)
	uphase(ii)=radinv*atan2(aimag(u0),real(u0))
	wphase(ii)=radinv*atan2(aimag(w0),real(w0))
			end do

	call ppplot

	STOP
	END

	subroutine ppplot

        common /amppha/ Yhc(5000),Yvc(5000),XX(5000),uphase(5000),
     1wphase(5000)
	common /param/ h,alpha1,beta1,rho1,alpha,beta,rho,ips,isurf,nper
     1,f,e
	common /splot/ wl,wr,wb,wt,xxl,xxr
	COMMON /plot/ XL,XR,YB,YT,FC,NN,IC,px,ppy,delinv
	character*9 char2*60,charf*9
        call dashdc('$''''''$',2,5)

        xxl=xl
        xxr=xr

	amin=999999
	amax=0

	do  i=1,nper
	if(yvc(i).gt.amax)then
	imax=i
	amax=yvc(i)
	go to 70
	end if 
	if(yvc(i).lt.amin)then
	imin=i
	amin=yvc(i)
	end if
70	end do

	do  i=1,nper
	if(yhc(i).gt.amax)then
	imax=i
	amax=yhc(i)
	go to 75
        end if
	if(yhc(i).lt.amin)then
	imin=i
	amin=yhc(i)
	end if
75	end do

	yb=amin
	if(yb.lt.1.e-3)yb=0
	yt=amax
	xl=alog10(xl)
	xr=alog10(xr)
	PX=.5*(XL+XR)
	PPY=.5*(YB+YT)

        CALL OPNGKS   
	CALL PLCHHQ(.5,.965,'Output of p-sv-layer.f',10.,0,0.)      
        CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET
	WB=.2
	WT=.7
	WL=.1
	WR=.7

c	print*,' Enter Fig. number:'
c	read(5,*)numfig
c	charf='Fig. 6-  '
c	write(charf(8:9),'(i2)')numfig
c        CALL PLCHHQ(.4,.1,charf,10.,0,0.)
        call lwidth(1000)
	CALL PLCHHQ(.4,.925,'Layer over a half-space',10.,0,0.)
	if(ips.eq.0)then
        CALL PLCHHQ(.4,.9,'Incident SV wave',10.,0,0.)
	ang=f
	else
        CALL PLCHHQ(.4,.9,'Incident P wave',10.,0,0.)
	ang=e
	end if

	char2='                                                            '
	char2(1:27)=':0701:, :0702:, :0721::333:'
	write(char2(29:32),'(f4.2)')alpha
	write(char2(35:38),'(f4.2)')beta
	write(char2(41:44),'(f4.2)')rho
	char2(33:33)=','
	char2(39:39)=','
	CALL PLCHHQ(.4,.825,char2(1:44),10.,0,0.)
	char2='                                                            '
	char2(1:43)=':0701::0743:,:0702::0743:,:0721::0743::333:'
	write(char2(45:48),'(f4.2)')alpha1
	write(char2(51:54),'(f4.2)')beta1
	write(char2(57:60),'(f4.2)')rho1
	char2(49:49)=','
	char2(55:55)=','
	CALL PLCHHQ(.4,.85,char2,10.,0,0.)
	char2='                                                            '
        char2(1:6)='h:333:'
        write(char2(8:11),'(f4.1)')h
        char2(13:14)='km'
        char2(23:28)='f:333:'
	if(ips.eq.1)char2(23:28)='e:333:'
        write(char2(30:33),'(f4.1)')ang
	char2(34:42)=':S1::632:'
        CALL PLCHHQ(.4,.8,char2(1:41),10.,0,0.)

	call plchhq(.54,.69,':PRU:u:B1:0:743: :333:',10.,0,0)	!u0
	call plchhq(.54,.66,':PRU:w:B1:0:743: :333:',10.,0,0)	!w0
        call lwidth(1000)
	dy=.01
	call line(.56,.7-dy,.63,.7-dy)
	call lined(.56,.69-dy,.63,.69-dy)
        call lwidth(3000)
	call line(.56,.67-dy,.63,.67-dy)
	call lined(.56,.66-dy,.63,.66-dy)

        call pplot(xl,xr,yb,yt,xll,ybb,xrr,hx)

	call lwidth(1000)
		call curve(xx,yhc,nper)
	call lwidth(3000)
		call curve(xx,yvc,nper)

c Plot phases. Scale the vertical axis
        DELY=800/(YT-YB)
        DELINV=1./DELY 
	do i=1,nper
        wphase(I)=yb+DELINV*(wphase(I)+400)
        uphase(I)=yb+DELINV*(uphase(I)+400)
        end do
	call curved(xx,wphase,nper)
        call lwidth(1000)
	call curved(xx,uphase,nper)

	call axes
1000	call frame
	call clsgks
	return
	END
c--------------------------------------------------------------------------------
	subroutine pplot(xl,xr,yb,yt,xll,ybb,xrr,hx)
	common /splot/ wl,wr,wb,wt,xxl,xxr
	character*3 char

        call lwidth(1000)
	dx=xr-xl
	dy=yt-yb
	xll=xl-.13*dx
	xrr=xr+.13*dx
	ybb=yb-.13*dy
	CALL SET(WL,WR,WB,WT,XLl,XRr,YBb,YT,1)  

	hx=.02*dx
	hy=.025*dy
	hx2=hx/2
	hy2=hy/2

	call plchhq(.5*(xll+xrr),yb-3.5*hy,'Period (sec)',10.,0,0)

		igo=0
		if(igo.eq.0)go to 15
C Period tick marks
	delx=.5
	if(dx.gt.10)delx=5

	ii=0
	do i=1,40
	xp=(i-1)*delx
	if(xp.lt.xl)go to 10
	if(xp.gt.xr)go to 20
	ii=ii+1
	call line(xp,yb,xp,yb-hy2)
	
	if(ii.eq.2*(ii/2))then
	call line(xp,yb,xp,yb-hy)
	call cchar(xp,char,ic)
	call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0) 
	end if
10	end do

C Period tick marks (logarithmic)
15      do i=1,9
        xp=i*.1 
        if(xp.lt.xxl)go to 12
        if(xp.gt.xxr)go to 13
        write(char(1:3),'(f3.1)')xp
        xp=alog10(xp)
        if(i.eq.2*(i/2).and.i.lt.8)
     1call plchhq(xp,yb-2*hy,char(1:3),10.,0,0)
        call line(xp,yb,xp,yb-hy)
12      end do
 
13     	do i=1,9
	xp=i
	if(xp.lt.xxl)go to 16
	if(xp.gt.xxr)go to 17
	call cchar(xp,char,ic)
	xp=alog10(xp)
	if(i.ne.2*(i/2).and.i.lt.8)call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0) 
	call line(xp,yb,xp,yb-hy)
16	end do
17	ii=0
	do i=10,100,10
	ii=ii+1
	xp=i
	if(xp.gt.xxr)go to 20
	call cchar(xp,char,ic)
	xp=alog10(xp)
	if(ii.ne.2*(ii/2))call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0) 
	call line(xp,yb,xp,yb-hy)
	end do

C Amplitude tick marks
20	dely=.5
	if(dy.gt.10)dely=5
	
	do i=1,40
	yp=(i-1)*dely
	if(yp.lt.yb)go to 30
	if(yp.gt.yt)go to 40
	call line(xl,yp,xl-hx2,yp)
30	end do

40	do i=1,40
	yp=2*(i-1)*dely
	if(yp.lt.yb)go to 35
	if(yp.gt.yt)go to 45
	call line(xl,yp,xl-hx,yp)
	call cchar(yp,char,ic)
	call plchhq(xl-2.5*hx,yp,char(1:ic),10.,0,0) 
35	end do

45	call line(xl,yb,xr,yb)
	call line(xl,yb,xl,yt)
	call line(xr,yb,xr,yt)
	return
	end 
c------------------------------------------------------------------------------
	subroutine lwidth(LINEWIDTH)
	CALL SETUSV('LW',LINEWIDTH)
	return
	end
c------------------------------------------------------------------------------
	subroutine cchar(xp,char,ic)
	character*3 char
	ip=xp
	char='   '
	if(0.le.ip.and.ip.le.9)then
	ic=1
	write(char(1:1),'(i1)')ip
	elseif(10.le.ip.and.ip.le.99)then
	ic=2
	write(char(1:2),'(i2)')ip
	elseif(100.le.ip.and.ip.le.999)then
	ic=3
	write(char(1:3),'(i3)')ip
	end if

	return
	end
C-------------------------------------------------------------------------------
	subroutine axes
	dimension cchar(5),ind(5)
	COMMON /plot/ XL,XR,YB,YT,FC,NN,IC,px,ppy,delinv
	CHARACTER*24 CNUM,cchar

	ind(1)=12
	cchar(1)=':0246::0720:'		!-pi
	ind(2)=24
	cchar(2)=':0246::0720::0517::0635:'	!-pi/2
	ind(3)=6
	cchar(3)=':0633:'			!0
	ind(4)=18
	cchar(4)=':0720::0517::0635:'	!pi/2
	ind(5)=6
	cchar(5)=':0720:'			!pi

	LINEWIDTH=1000
	CALL SETUSV('LW',LINEWIDTH)
	CALL LINE(XL,YB,XR,YB)
	CALL LINE(XL,YT,XR,YT)
	CALL LINE(XL,YB,XL,YT)
	CALL LINE(XR,YB,XR,YT)
c	CALL PLCHHQ(XL-.17,PPY,'Amplitude',10.,90.,0.)
	CALL PLCHHQ(XL-.17,PPY,'Absolute value',10.,90.,0.)

	CALL PLCHHQ(XR+.18,PPY,'Phase',10.,90.,0.)
	DO I=1,5
	IANG=-180+(I-1)*90	
	ANG=yb+DELINV*(iang+400)
	CNUM(1:ind(i))=cchar(i)
	CALL PLCHHQ(XR+.06,ANG,CNUM(1:ind(i)),10.,0,-1.)
	CALL LINE(XR,ANG,XR+.03,ANG)
	END DO

	return
	end

	SUBROUTINE EQUATION(A,X,N)
	DIMENSION A(6,7),X(6) 
	complex a,x,f

	NP=N+1
	DO 9 I=1,N
	IP=I+1
	DO 9 J=1,N
	IF(I-J)4,9,4
4	F=-A(J,I)/A(I,I)
	DO 3 K=IP,NP
3	A(J,K)=A(J,K)+F*A(I,K)
9	CONTINUE
	DO 10 I=1,N
	X(I)=A(I,N+1)/A(I,I)
10	CONTINUE
	RETURN
	END
