C-------------------------------------------------------------------------------
C Input: fault parameters: strike, dip, slip of fault plane
C Output: the azimuth (from north) and dip of the p, t, and b axes
C The components of the three axes are stored in the variables p1,p2,p3; t1,t2,t3;
C b1,b2,b3
C Equations (10.7.12), (10.7.15), (10.7.16), (10.7.18) are used.
C Author: J. Pujol

        subroutine pbt(STRIKE,DIP,SLIP,az1p,dpp,az1t,dpt,az1b,dpb)
	COMMON /AZDP/ AZ,AZ1,DP,DEGRAD
	CHARACTER*1 AXIS(3)
        AXIS(1)='P'
        AXIS(2)='B'
        AXIS(3)='T'
	PI4=ATAN(1.)
	DEGRAD=PI4/45
c	write(3,20)STRIKE,DIP,SLIP
   20 FORMAT(////,' STRIKE, DIP AND SLIP (IN DEGREES):',3F10.1,//)
        ALAMBDA=SLIP*DEGRAD
	DELTA=DIP*DEGRAD
	FI=STRIKE*DEGRAD
        SL=SIN(ALAMBDA)
        CL=COS(ALAMBDA)
	SD=SIN(DELTA)
	CD=COS(DELTA)
	SF=SIN(FI)
	CF=COS(FI)
	R2=1/SQRT(2.)
	A1=SL*CD
	A=A1-SD
	E=A1+SD
	C=CL*CF
	D=CL*SF
	B=CL*CD
	F=SL*SD
	T1=-R2*(C+A*SF)
	T2=R2*(D-A*CF)
	T3=R2*(F+CD)
	B1=SL*CF-B*SF
	B2=-(SL*SF+B*CF)
	B3=CL*SD
	P1=-R2*(C+E*SF)
	P2=R2*(D-E*CF)
	P3=R2*(F-CD)
c	write(3,40)P1,P2,P3,B1,B2,B3,T1,T2,T3
   40 FORMAT(' P1,P2,P3',3F10.2,/,' B1,B2,B3',3F10.2,/,' T1,T2,T3',3
     &F10.2,/)
   30 FORMAT('  AXIS      AZ(FROM S)     AZ(FROM N)        DIP',/)
      AMODP=SQRT(P1*P1+P2*P2+P3*P3)
      AMODB=SQRT(B1*B1+B2*B2+B3*B3)
      AMODT=SQRT(T1*T1+T2*T2+T3*T3)
        COSPT=(P1*T1+P2*T2+P3*T3)/(AMODP*AMODT)
        COSPB=(P1*B1+P2*B2+P3*B3)/(AMODP*AMODB)
        COSTB=(T1*B1+T2*B2+T3*B3)/(AMODT*AMODB)

        if(abs(cospt).gt.1.e-5.or.abs(cospb).gt.1.e-5
     1.or.abs(costb).gt.1.e-5)then
        print*,' SUBROUTINE pbt-sub: P, B, T axes not perpendicular!!!'
        end if

	CALL ANGLES(P1,P2,P3)
	az1p=az1
	dpp=dp
	CALL ANGLES(B1,B2,B3)
	az1b=az1
	dpb=dp
	CALL ANGLES(T1,T2,T3)
	az1t=az1
	dpt=dp

	return
	END
	SUBROUTINE ANGLES(A1,A2,A3)
	COMMON  /AZDP/ AZ,AZ1,DP,DEGRAD
	ZERO=1E-7
	RAD=1/DEGRAD
	AA1=A1*A1
	AA2=A2*A2
	AA3=A3*A3
	SA1=AA1+AA2
	SA2=SQRT(SA1+AA3)
	SA=SQRT(SA1)
        a1sa=a1/sa
C The following commented out statements were needed for some special cases 
C when using the subroutine in a PC under Windows in a DOS emulation. The 
C argument of the cosine was larger than one in absolute value.
c                if(a1sa.gt.1.)then
c                a1sa=1
c                elseif(a1sa.lt.-1.)then
c                a1sa=-1
c                end if
        AZ=(ACOS(A1SA))*RAD
      IF(A2.LT.0.)AZ=360-AZ
      AZ1=180-AZ
        sasa2=sa/sa2
c                if(sasa2.gt.1.)then
c                sasa2=1
c                elseif(sasa2.lt.-1.)then
c                sasa2=-1
c                end if
        DP=RAD*(ACOS(SASA2))
      IF(A3.GT.ZERO)AZ1=-AZ
	IF(AZ1.LT.0.)AZ1=AZ1+360
	RETURN
	END
