C Program radproj-n.f
C Plot of radiation patterns given a set of fault parameters or moment tensor
C components. Lower-hemisphere equal-area projection.
C Based on programs written by B. Kennett (see Kennett, B., 1988, Radiation from 
C a moment-tensor source, in: D. Doornbos, Ed., Seismological algorithms, Academic 
C Press, 427-441.) The original programs can be downloaded from
C http://orfeus.knmi.nl/other.services/software.shtml (click on Seismological algorithms) 
C The conversion to NCAR and the polarization angle, fault planes and P, B, T axes 
C are due to J.P.
C The output file fort.1 lists the largest singular value lambda1, the singular values
C normalized by lambda1, and the P, B, T axes.
C To compile and link:
C f77 -c eigen2.f
C f77 -c nodplane-sub.f
C ncargf77 radproj-n.f eigen2.o nodplane-sub.o  -o radproj-n
C NOTE 1:
C The fault planes option does not work when strike = 180; dip= 0, 90; slip= 0, 180. 
C The other features appear to work fine for these angles. Not all the combinations
C have been tested. The fault planes for values of strike, dip, slip close to these 
C special values are OK. The following inputs work fine
C 4 <CR> 0 <CR> 0 <CR> 180   90   180    <CR>	         (<CR> = carriage return) 
C 4 <CR> 2 <CR> 0 <CR> 179.8 89.8 179.8  <CR>
C 4 <CR> 0 <CR> 0 <CR>   0    0     0    <CR>
C 4 <CR> 2 <CR> 0 <CR>   0    0.1   0.1  <CR>
C NOTE 2: the following inputs give the same moment tensor 0 0 1, 0 0 0, 1 0 0 (M13+M31) 
C 4 <CR> 0 <CR> 0 <CR> 180    0     0    <CR>
C 4 <CR> 0 <CR> 0 <CR>   0    0   180    <CR> 
C C In both cases Fig. 10.7 is obtained. 
C Fig. 10.7 can also be obtained entering the moment tensor
C 4 <CR> 1 <CR> 0 <CR> 0 0 1 0 0 0 1 0 0 <CR>
C See the NOTE in subroutine diag

	dimension xp(500),yp(500),angle(500),sz(500),fp(500),fs(500),
     1fh(500),azim(500),xtbp(3),ytbp(3),chstr(3)
	COMMON /DDIAG/ AMT(3,3),AZ(3),DDIP(3)
	COMMON /fFOCAL/ STRK(2),FDIP(2),SLIP(2)
      common /info/ strike,dip,rake,st,di,ra                       
      character*1 IANS,ISIZ,chstr*6,cfig*6,chart*39
c	print*,' Enter fig. number, (a5):'
c	read(5,'(a5)')cfig
	ppi=4*atan(1.)
	radpi=ppi/180
      DGR = radpi
        CALL OPNGKS 
	CALL PLCHHQ(.5,.985,'Output of radproj-n.f',10.,0,0.)        
c	if(cfig.ne.'     ')call plCHHQ(.5,.05,'Fig. '//cfig,10.,0,0.)

	PRINT*,' ENTER 1, 2, 3 FOR P, SV, SH, RESPECTIVELY; 4 FOR ALL 
     1OF THEM OR FOR SV, SH ONLY:'
	READ(5,*)IPS
	PRINT*,' Enter 0 for double couple; 1 M.T.; 2 double couple with
     1 fault planes plotted (see Fig. 10.9):'
	read(5,*)iansW
	PRINT*,' Enter 1 for arrows (SH and SV); 2 for SH and SV only;
     1 0 for + and - (see Figs. 10.9, 10.10):'
	read(5,*)iarrow
	IF(IANSW.EQ.0)THEN
	IANS='D'
	ELSEIF(IANSW.EQ.1)THEN
	IANS='M'
	ELSEIF(IANSW.EQ.2)THEN
	IANS='F'
	END IF
      IF(IANS.EQ.'D') THEN                                              
	print*,' Enter STRIKE, DIP, SLIP:'
        READ(5,*) STRIKE,DIP,RAKE                                       
        WRITE(6,*) STRIKE,DIP,RAKE                                      
      ELSEIF(IANS.EQ.'F') THEN
	print*,' Enter STRIKE, DIP, SLIP:'	
	READ(5,*)STRK(1),FDIP(1),SLIP(1)
        call nodplane(STRK(1),FDIP(1),SLIP(1),STRK(2),FDIP(2),SLIP(2))
        STRIKE=STRK(1)
	DIP=FDIP(1)
	RAKE=SLIP(1)
        WRITE(6,*) STRIKE,DIP,RAKE                                      
      ELSE IF (IANS.EQ.'M') THEN                                        
        Print*,' Enter moment tensor components (row-wise). Aki and
     1Richards (1980) convention used:'
        READ(5,*)((amt(i,j),j=1,3),i=1,3) 
      END IF                                                            

      ST = DGR*STRIKE                                                   
      DI = DGR*DIP                                                      
      RA = DGR*RAKE                                                     

	if (ians.eq.'D'.or.ians.eq.'F')then
	 call momang  
                              
 	chart='Strike=      ;  Dip=     ;  Slip=      '
	write(chart(8:13),'(F6.1)')strike
	write(chart(21:25),'(F5.1)')dip
	write(chart(34:39),'(F6.1)')rake
	CALL PLCHHQ(.5,.96,chart,11.,0,0.) 
	end if

	YB=0
	YT=1
	xl=0
	xr=1

	WT=.9
	WB=0.1
	WL=0.1
	WR=.9

	LINEWIDTH=1500
	CALL SETUSV('LW',LINEWIDTH)
	ISYMBSIZ=100
	CALL SETUSV('MS',ISYMBSIZ)

	CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET

	CALL SET(WL,WR,WB,WT,XL,XR,YB,YT,1)  

	LINEWIDTH=2000
	CALL SETUSV('LW',LINEWIDTH)

	call diag
      write (1,34)                                                     
34    format (/,' axes:  az.   dip')                                   
      write (1,35) (az(n),ddip(n),n=1,3)                                
	write(1,103)
103	format(/,' azimuth cw from north, dip down from horiz. plane')
35    format (' T: ',2F7.1,/,' B: ',2F7.1,/,' P: ',2F7.1)     

	rad=0.2
      PRAD = RAD*SQRT(2.0)                                              
	do i=1,3			!Projection of the T, B, P axes
	pr=prad*sin(radpi*(45-0.5*ddip(i)))	!CHECK 45-
	ytbp(i)=pr*cos(radpi*az(i))
	xtbp(i)=pr*sin(radpi*az(i))
	end do
	chstr(1)=':1245:'	!dot
	chstr(2)=':0541:'	!star
	chstr(3)=':1331:'	!circle

C      DRAW SOURCE RADIATION PATTERNS                                   
	GO TO (1000,2000,3000,1000),IPS
C                                      P WAVES                          
1000	x0=0.25
	if(iarrow.eq.2)go to 2000
	y0=0.75
	if(ians.eq.'F')call focal (x0,y0,rad)
	call circle(x0,y0,rad)
	CALL SETUSV('LW',3000)
	call plCHHQ(x0+rad,y0+rad,'P',25.,0,0.)
	CALL SETUSV('LW',2000)
	i=0
	do 10 PHI = 0.0,360.0,10.0					!
        PH = DGR*PHI                                                    
        PHST = PH - ST                                                  
        do 20 TXI =  5.0,86.0,10.0                                      
	i=i+1
	azim(i)=ph
          TX = DGR*TXI                                                  
            FA = COS(PH)*COS(PH)*amt(1,1)+2.0*SIN(PH)*COS(PH)*amt(1,2)        
     &           + SIN(PH)*SIN(PH)*amt(2,2) - amt(3,3)                          
            FB = COS(PH)*amt(1,3) + SIN(PH)*amt(2,3)                            
            FP(i) = SIN(TX)*SIN(TX)*FA+2.0*SIN(TX)*COS(TX)*FB+amt(3,3)     
c                              equal area projection                    
          PR = PRAD*SIN(0.5*TX)                                         
          YP(i) =PR*COS(PH)                                               
          XP(i) =PR*SIN(PH)                                               
          angle(i) = 0.0                                                   
	sz(i)=abs(fp(i))
 20     continue                                                        
 10   continue
	nn=i
	do j=1,3
	call plCHHQ(x0+xtbp(j),y0+ytbp(j),chstr(j),20.,0,0.)
	end do

		do j=1,nn
          IF(FP(j).GT.0.0) then                                            
	call plus(x0+xp(j),y0+yp(j),sz(j))
          ELSE IF (FP(j).LT.0.0) then                                      
	call minus(x0+xp(j),y0+yp(j),sz(j))
          END IF                                                        
		end do

	IF(IPS.EQ.4)THEN
	GO TO 2000
	ELSE
	GO TO 4000
	END IF
C                                      SV WAVES                          
2000	x0=0.75
	y0=0.25
	if(ians.eq.'F'.and.iarrow.eq.0)call focal (x0,y0,rad)

	call circle(x0,y0,rad)
	CALL SETUSV('LW',3000)
	call plCHHQ(x0+rad,y0+rad,'SV',25.,0,0.)
	CALL SETUSV('LW',2000)
	i=0
	do 30 PHI = 0.0,360.0,10.0                                       
        PH = DGR*PHI                                                     
        PHST = PH - ST                                                   
        do 40 TXI =  5.0,86.0,10.0                                       
	i=i+1
          TX = DGR*TXI                                                   
            FA = COS(PH)*COS(PH)*amt(1,1)+2.0*SIN(PH)*COS(PH)*amt(1,2)         
     &           + SIN(PH)*SIN(PH)*amt(2,2) - amt(3,3)                           
            FB = COS(PH)*amt(1,3) + SIN(PH)*amt(2,3)                            
            FS(i) = SIN(TX)*COS(TX)*FA+(1.0-2.0*SIN(TX)*SIN(TX))*FB      
c                              equal area projection                     
          PR = PRAD*SIN(0.5*TX)                                          
          YP(i) =PR*COS(PH)                                               
          XP(i) =PR*SIN(PH)                                               
	sz(i)=abs(fs(i))
          IF(FS(i).GT.0.0) then                                             
          ANGLE(i) = 1.5707963-ph                                           
          ELSE IF(FS(i).LT.0.0) then                                        
          ANGLE(i)=4.7123889-ph                                           
          END IF                                                      
 40     continue                                                        
 30   continue                     
		nn=i
	do j=1,3
	call plCHHQ(x0+xtbp(j),y0+ytbp(j),chstr(j),20.,0,0.)
	end do

			if(iarrow.ge.1)then
		do j=1,nn
	call arrow(x0+xp(j),y0+yp(j),sz(j),angle(j))
		end do
			else
		do j=1,nn
          IF(Fs(j).GT.0.0) then                                            
	call plus(x0+xp(j),y0+yp(j),sz(j))
          ELSE IF (Fs(j).LT.0.0) then                                      
	call minus(x0+xp(j),y0+yp(j),sz(j))
          END IF                                                        
		end do
			end if

	IF(IPS.EQ.4)THEN
	GO TO 3000
	ELSE
	GO TO 4000
	END IF
C                                      SH WAVES                          
3000	x0=0.75
	y0=0.75

	ppi=4*atan(1.)
	pprad=180/ppi

	if(ians.eq.'F'.and.iarrow.eq.0)call focal (x0,y0,rad)

	call circle(x0,y0,rad)
	CALL SETUSV('LW',3000)
	call plCHHQ(x0+rad,y0+rad,'SH',25.,0,0.)
	CALL SETUSV('LW',2000)
	i=0
	do 50 PHI = 0.0,360.0,10.0                                       
        PH = DGR*PHI                                                     
        PHST = PH - ST                                                  
        do 60 TXI =  5.0,86.0,10.0                                     
	i=i+1
          TX = DGR*TXI                                                  
            FA =SIN(PH)*COS(PH)*(amt(2,2)-amt(1,1))+COS(2.0*PH)*amt(1,2)          
            FB = COS(PH)*amt(2,3) - SIN(PH)*amt(1,3)                             
            FH(i) = SIN(TX)*FA + COS(TX)*FB                                 

c                              equal area projection                     
          PR = PRAD*SIN(0.5*TX)                                         
          YP(i) =PR*COS(PH)                                                
          XP(i) =PR*SIN(PH)                                               
	sz(i)=abs(fh(i))
          IF(FH(i).GT.0.0) then                                             
            angle(i) = -ph                                                 
          ELSE IF(FH(i).LT.0.0) then                                      
            angle(i)=3.1415927-ph                                           
          END IF                                                        
 60     continue                                                        
 50   continue                                                           
		nn=i
	do j=1,3
	call plCHHQ(x0+xtbp(j),y0+ytbp(j),chstr(j),20.,0,0.)
	end do

			if(iarrow.ge.1)then
		do j=1,nn
	call arrow(x0+xp(j),y0+yp(j),sz(j),angle(j))
		end do
 			else
		do j=1,nn
c			write(7,32)j,fh(j)
32	format(i5,f8.3)
          IF(Fh(j).GT.0.0) then                                            
	call plus(x0+xp(j),y0+yp(j),sz(j))
          ELSE IF (Fh(j).LT.0.0) then                                      
	call minus(x0+xp(j),y0+yp(j),sz(j))
          END IF                                                        
		end do
			end if

4000	x0=.25
	y0=.25
	if(iarrow.eq.2)go to 5000
	CALL SETUSV('LW',3000)
	if(ips.gt.1)call plCHHQ(x0+rad,y0+rad,':0705:',35.,0,0.)
	CALL SETUSV('LW',2000)

		if(ips.eq.1)go to 5000
	do j=1,3
	call plCHHQ(x0+xtbp(j),y0+ytbp(j),chstr(j),20.,0,0.)
	end do
c Polarization
	call circle(x0,y0,rad)
	do j=1,nn,2
	if(abs(fh(j)).lt.1.e-5.and.abs(fs(j)).lt.1.e-5)go to 21
	ang=atan2(fh(j),fs(j))
	amp=sqrt(fh(j)**2+fs(j)**2)
	call polar(x0+xp(j),y0+yp(j),ang+azim(j),amp)
21	end do


5000	call frame
	call clsgks
      STOP                                                              
      END                                                               

      subroutine momang                                                 
C Compute moment tensor given fault parameters. Aki and Richards convention
C (see (10.7.11) and (10.7.21))
	COMMON /DDIAG/ AMT(3,3),AZ(3),DDIP(3)
      common /info/ strike,dip,rake,st,di,ra                       

      sd = sin(di)                                                      
      cd = cos(di)                                                      
      s2d = sin(2.0*di)                                                 
      c2d = cos(2.0*di)                                                 
      sr = sin(ra)                                                      
      cr = cos(ra)                                                      
      ss = sin(st)                                                      
      cs = cos(st)                                                      
      s2s = sin(2.0*st)                                                 
      c2s = cos(2.0*st)                                                 

      amt(1,1) = -(sd*cr*s2s + s2d*sr*ss*ss)                                
      amt(1,2) =  (sd*cr*c2s + s2d*sr*ss*cs)                                
      amt(1,3) = -(cd*cr*cs + c2d*sr*ss)                                    
      amt(2,1) = amt(1,2)                                                  
      amt(2,2) =  (sd*cr*s2s - s2d*sr*cs*cs)                                
      amt(2,3) = -(cd*cr*ss - c2d*sr*cs)                                    
      amt(3,1) = amt(1,3)                                                    
      amt(3,2) = amt(2,3)                                                  
      amt(3,3) =  s2d*sr                                                    

	print*,' Moment tensor components for Double Couple'           
	write(6,*)((amt(i,j),j=1,3),i=1,3)
      return                                                            
      end                                                               
c ==============================================================================
	subroutine circle(x0,y0,rad)
	parameter (np=361)
	dimension x(np),y(np)
	ppi=4*atan(1.)
	radpi=ppi/180
	do i=1,np
	ang=i
	x(i)=x0+rad*cos(radpi*ang)
	y(i)=y0+rad*sin(radpi*ang)
	end do
	call curve(x,y,np)

        call line(x(1),y(1),x0+1.04*rad,y(1))
        call plCHHQ(x0+1.1*rad,y0,'E',15.,0,0.)
        call line(x0-rad,y0,x0-1.04*rad,y0)
        call plCHHQ(x0-1.13*rad,y0,'W',15.,0,0.)
        call line(x0,y0+rad,x0,y0+1.04*rad)
	call plCHHQ(x0,y0+1.1*rad,'N',15.,0,0.)
        call line(x0,y0-rad,x0,y0-1.04*rad)
        call plCHHQ(x0,y0-1.15*rad,'S',15.,0,0.)

	return
	end
c ==============================================================================
	subroutine arrow(xp,yp,sz,angle)
c Based on the original subroutine bsymbol
        szc=0.0075*sz*cos(angle)
        szs=0.0075*sz*sin(angle)
        call line(xp,yp,xp+szc,yp+szs)
        call line(xp,yp,xp-szc,yp-szs)
        call line(xp-szs,yp+szc,xp-szs,yp+szc)
        call line(xp-szs,yp+szc,xp+szc,yp+szs)
        call line(xp+szc,yp+szs,xp+szs,yp-szc)
        return
	end
c ==============================================================================
	subroutine plus(xp,yp,sz)
c Based on the original subroutine bsymbol
        szz=0.0075*sz
        call line(xp-szz,yp,xp+szz,yp)
        call line(xp,yp-szz,xp,yp++szz)
        return
	end
c ==============================================================================
	subroutine minus(xp,yp,sz)
c Based on the original subroutine bsymbol
        szz=0.0075*sz
        call line(xp-szz,yp,xp+szz,yp)
        return
	end
c ==============================================================================
	subroutine polar(xp,yp,angle,sz)
c Based on the original subroutine bsymbol
        szs=0.0075*cos(angle)*sz
        szc=0.0075*sin(angle)*sz
        call line(xp,yp,xp+szc,yp+szs)
        call line(xp,yp,xp-szc,yp-szs)
        return
	end
c ==============================================================================
	SUBROUTINE DIAG
C Diagonalize moment tensor and determine the azimuth and dip of the P, B, T axes
      DIMENSION D(3),Z(3,3),e(3)
	COMMON /DDIAG/ AMT(3,3),AZ(3),DIP(3)
	ppi=4*atan(1.)
	pprad=180/ppi

 	call tred2(3,3,amt,d,e,z)
	call imtql2(3,3,d,e,z,ierr)
c	print*,' IERR:',ierr
c	WRITE(1,*)(Z(I,1),I=1,3)
c	WRITE(1,*)(Z(I,2),I=1,3)
c	WRITE(1,*)(Z(I,3),I=1,3)
c	WRITE (1,*) (D(N),N=1,3)

	CALL SORT(D,Z,3)

C Normalize moment tensor (needed for plotting). 
	do i=1,3
	do j=1,3
	amt(i,j)=amt(i,j)/d(1)
	end do
	end do	

C NOTE: in this version of the program a public domain subroutine replaces
C the original commercial subroutine. Initially, with the new routine the inputs
C 4 <CR> 0 <CR> 0 <CR> 180    0     0    <CR>
C 4 <CR> 1 <CR> 0 <CR> 0 0 1 0 0 0 1 0 0 <CR>
C produced the B axis plotted on W, rather than on E (see Fig. 10.7). To make sure
C that the B axis is correctly determined we use  b = t x p [x = vector product; see 
C (9.11.6)]. Vectors are stored column-wise:
	z(1,2)=z(2,1)*z(3,3)-z(3,1)*z(2,3)
	z(2,2)=z(3,1)*z(1,3)-z(1,1)*z(3,3)
	z(3,2)=z(1,1)*z(2,3)-z(2,1)*z(1,3)

	WRITE(1,25)D(1)
25	FORMAT(' LAMBDA1:',E10.3)
	WRITE (1,20) (D(N)/D(1),N=1,3)
20    FORMAT (' LAMBDAS/LAMBDA1: T=',F10.3,' B=',F10.3,' P=',F10.3)

      DO 30 N=1,3  
      AZ(N)=ATAN2(Z(2,N),Z(1,N))*pprad
      A1=SQRT(Z(1,N)*Z(1,N)+Z(2,N)*Z(2,N))
      A2=ABS(Z(3,N))                      
      DIP(N)=ATAN2(A2,A1)*pprad
      IF (Z(3,N).LT.0.) AZ(N)=AZ(N)-180   
30	IF(AZ(N).LT..0)AZ(N)=AZ(N)+360
      RETURN                          
      END                             
c==============================================================================
C	SORT SINGULAR VALUES (descending order)
	SUBROUTINE SORT(S,U,NC)
	DIMENSION S(3),U(3,3)

	DO 45 K=1,NC
	G=-1.e+40	!THIS VALUE MUST BE SUFFICIENTLY LARGE IN ABS. VALUE 
	DO 39 I=K,NC
		IF(S(I).LT.G)GO TO 39
		G=S(I)
		J=I
39	CONTINUE
	IF(J.EQ.K)GO TO 45
	S(J)=S(K)
	S(K)=G
	DO 140 I=1,NC
	Q=U(I,J)
	U(I,J)=U(I,K)
140	U(I,K)=Q
45	CONTINUE

	RETURN
	END
C============================================================================
	SUBROUTINE FOCAL(X0,Y0,RC)
C PLOT FOCAL PLANES
	DIMENSION XC(500),YC(500)
	COMMON /fFOCAL/ STRK(2),DDIP(2),SLIP(2)
	PI4=ATAN(1.)
	RAD=PI4/45
	SQR2=SQRT(2.)
	PI2=2*PI4
		DO K=1,2
	STRIKE=STRK(K)*RAD
	DIP=DDIP(K)*RAD
	SLP=SLIP(K)
	IF(SLP.GT.180)SLP=SLP-180
	ISL=0
	IF(SLP.GT.90)THEN
	SLP=180-SLP
	ISL=1
	END IF
	SLP=SLP*RAD
	DO I=1,361
	AZIM=((I-1)/2.)*RAD
	APDIP=ATAN(TAN(DIP)*SIN(AZIM))
	RL=SQR2*RC*SIN(PI4-APDIP/2.)
	XC(I)=X0+RL*SIN(AZIM+STRIKE)
	YC(I)=Y0+RL*COS(AZIM+STRIKE)
	END DO
	call curve(xc,yc,361)

c PLOT SLIP VECTORS
 	TANS=TAN(SLP)
	AZMS=ATAN(TANS*COS(DIP))
	IF(ISL.EQ.0)AZMS=180*RAD-AZMS
	SIND=SIN(DIP)
	SINS=SIN(SLP)
	SINDS2=SINS*SINS*SIND*SIND
	DIPS=(1-SINDS2)/SINDS2
	DIPS=SQRT(DIPS)
	DIPS=ATAN(DIPS)
	RL=SQR2*RC*SIN(DIPS/2.)
	X=RL*SIN(AZMS+STRIKE)
	Y=RL*COS(AZMS+STRIKE)
	call plCHHQ(x+x0,y+y0,':1246:',15.,0,0.)	!square
		END DO

	RETURN
	END

