C Program synth-pnt-src.f  
C Author: J. Pujol
C Synthetic seismograms for a point source (total field).
C The arrays G1, G2, G3 correspond to terms that depend on R, R**2 and
C R**3 respectively in (9.5.19). The first index corresponds to time and 
C the other two to i and j.  
C The peak amplitudes given here are for the full synthetics, represented
C by G = G1 + G2 + G3
C The arrays DEL and GA correspond to delta(i,j) and gamma(i).
C The arrays AJ0, AJ1, AJ2 correspond to the functions J, J', J'' in
C (9.5.21)-(9.5.23)

	PARAMETER (NS=1024)
	DIMENSION AJ0(NS),AJ1(NS),AJ2(NS),TI(NS),DEL(3,3),GA(3),Y(NS),
     1G(NS,3,3),G1(NS,3,3),G2(NS,3,3),G3(NS,3,3),ts(ns),y2(ns),afa(2),
     1rdis(2),ddd(2),yg(NS)
	character*2 cnum,cnum2*7,cchar*3,cchar2*19,cchara*1

	NP=650

C VECTOR GAMMA:
	GA(1)=.75
	GA(2)=.433
	GA(3)=.5

	VP=6*1.E+5	!KM/S; KM CONVERTED TO CM
	VS=3.5*1.E+5	!KM/S; KM CONVERTED TO CM
	RHO=2.8
	PI=4*ATAN(1.)
	RHOPI=1./(4*PI*RHO)
	R=100*1.E+5	!KM CONVERTED TO CM
	DT=0.1	!SEC
	ndi=np*dt

        CALL OPNGKS        
c	CALL PLCHHQ(.9,.02,'Fig. 9-4',10.,0.,0)
	CALL PLCHHQ(.5,.965,'Output of synth-pnt-src.f ',10.,0,0.)
	XL=0
	xr=3*np*dt
	xlr=.5*(xl+xr)
	YB=-4.3
	YT=5.5

	XL=XL-3
	XR=XR+3
	
	WT=1
	WB=.0	
	WL=.2
	WR=.8

	ISYMBSIZ=100
	CALL SETUSV('MS',ISYMBSIZ)

	CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET
	CALL SET(WL,WR,WB,WT,XL,XR,YB,YT,1)  

	LINEWIDTH=2000
	CALL SETUSV('LW',LINEWIDTH)
	
	afa(1)=1
	afa(2)=3	!2
	ddd(1)=4
	ddd(2)=1
	rdis(1)=100.e+5
	rdis(2)=200.e+5
	dfac=0
	n2=0
							do jjj=1,2
	if(jjj.eq.2)dfac=6
	r=rdis(jjj)
	if(jjj.eq.2)n2=(rdis(1)/VP)/DT
						do kkk=1,2
	if(jjj.eq.2.and.kkk.eq.2)go to 1000

	dd1=ddd(kkk)-dfac
	a=afa(kkk)
	AI=1./A
	AA=A*A

	DO I=1,NS
	T=(I-1)*DT
	Ts(I)=T
	AT=A*T
	EAT=EXP(-AT)
	AJ1(I)=1-EAT-AT*EAT
	AJ0(I)=T-AI*(1+AJ1(I)-EAT)
	AJ2(I)=AA*T*EAT
	END DO

C Generate the synthetics
	RI3=1./(R**3)
	RI2=1./(R**2)
	RI=1./R
	VPI=1./VP
	VSI=1./VS
	VP2I=VPI**2
	VS2I=VSI**2
	NRVP=(R/VP)/DT
	NRVS=(R/VS)/DT
	DELN=NRVS-NRVP
 
	DO I=1,3
	DO J=1,3
	DEL(I,J)=0	
	END DO
	END DO
	DO J=1,3
	DEL(J,J)=1	!DELTA
	END DO

	DO k=1,ns
	DO I=1,3
	DO J=1,3
	g(k,i,j)=0
	g1(k,i,j)=0
	g2(k,i,j)=0
	g3(k,i,j)=0
	end do
	end do
	end do

	DO I=1,3
	DO J=1,3
	IF(I.GT.J)GO TO 10
	GIGJ=GA(I)*GA(J)
	GIGJD=GIGJ-DEL(I,J)
	GIG3=3*GIGJ-DEL(I,J)	
	FACP1=VP2I*GIGJ*RHOPI
	FACS1=VS2I*GIGJD*RHOPI	
	FAC2=RI2*GIG3*RHOPI
	FAC3=RI3*GIG3*RHOPI

	DO K=NRVP,NS
	IP=K-NRVP+1
	G1(K,I,J)=RI*FACP1*AJ2(IP)
	G2(K,I,J)=FAC2*VPI*AJ1(IP)
	G3(K,I,J)=FAC3*AJ0(IP)
	G(K,I,J)=G1(K,I,J)+G2(K,I,J)+G3(K,I,J)
	END DO

	DO K=NRVS,NS
	IS=K-NRVS+1
	C1=RI*FACS1*AJ2(IS)
	C2=FAC2*VSI*AJ1(IS)
	C3=FAC3*AJ0(IS)
	G1(K,I,J)=G1(K,I,J)-C1
	G2(K,I,J)=G2(K,I,J)-C2
	G3(K,I,J)=G3(K,I,J)-C3
	G(K,I,J)=G(K,I,J)-(C1+C2+C3)
	END DO
10	END DO
	END DO

			do ii=1,3
			do jj=1,3
	if(ii.gt.jj)go to 200
	dd=dd1
	if(.not.(ii.eq.jj))dd=dd1-1	
	DO I=1,NP
	Yg(I)=G(I,II,JJ)
	Y(I)=G1(I,II,JJ)
	END DO

	AMAX=0
	DO I=1,NP
	ABSY=ABS(Y(I))
	IF(AMAX.LT.ABSY)AMAX=ABSY
	END DO

	AMAXg=0
	DO I=1,NP
	ABSYg=ABS(Yg(I))
	IF(AMAXg.LT.ABSYg)AMAXg=ABSYg
	END DO
c	write(6,*)AMAXg
	write(cnum2,20)AMAXg
20	format(1pe7.1)

C The following was used to plot the different contributions to the total field
	IGO=0
	IF(IGO.EQ.0)GO TO 130
	IF(IG.EQ.1)THEN
	DO I=1,NP
	Y(I)=G1(I,II,JJ)
	END DO
	ELSEIF(IG.EQ.2)THEN
	DO I=1,NP
	Y(I)=G2(I,II,JJ)
	END DO
	ELSEIF(IG.EQ.3)THEN
	DO I=1,NP
	Y(I)=G3(I,II,JJ)
	END DO
	END IF

130	DO I=1,NP
	Y(I)=G(I,II,JJ)
	END DO

	off=ndi*(ii-1)
	if(ii.ne.jj)off=(ii+jj-3)*ndi
	amaxi=0.5/amax
	DO I=1,NP
	y2(i)=0
	Y2(I)=dd+amaxi*Y(I+n2)
	ti(i)=off+ts(i)
	END DO
	num=ii*10+jj
	write(cnum,5)num
5	format(i2)
	CALL PLCHHQ(ti(10),dd+.5,cnum,10.,0.,-1.)
	CALL SETUSV('LW',1500)
	CALL PLCHHQ(ti(10),dd+.33,cnum2,8.,0.,-1.)
	CALL SETUSV('LW',2000)

	if(num.eq.11)then
	ddt=0
	if(jjj.eq.2)ddt=rdis(1)/vp
	CALL PLCHHQ(ti(nrvp)-ddt,dd-.12,':1134:',20.,0.,0.)
	CALL PLCHHQ(ti(nrvs)-ddt,dd-.12,':1134:',20.,0.,0.)
	end if
	
	CALL SETUSV('LW',1500)
	if(jjj.eq.2.and.ii.eq.jj)then
	call line(ti(1),-3.95,ti(np-50),-3.95)
	do i=1,6
	t10=i*10+off
	i10=i*10	
	write(cnum,5)i10
	if(i.lt.6)CALL PLCHHQ(t10,-3.78,cnum,8.,0.,0.)
	call line(t10,-3.95,t10,-3.88)
	end do
	cchara='0'
	CALL PLCHHQ(off,-3.78,cchara,8.,0.,0.)
	call line(off,-3.95,off,-3.88)
	CALL PLCHHQ(.5*(ti(1)+ti(np-50)),-4.1,'Time (s)',10.,0.,0.)
	end if

	CALL SETUSV('LW',2000)
	if(num.eq.33)then
	ir=r/1.e+5
	write(cchar,6)ir	
6	format(i3)
	ia=a
	write(cchara,7)ia	
7	format(i1)
	cchar2='r = '//cchar//' km    a = '//cchara
	CALL PLCHHQ(xlr,dd+.9,cchar2,10.,0.,0.)
	end if
	CALL SETUSV('LW',3000)
	CALL CURVE(TI,Y2,np-50)

	DO I=1,NP
	Y(I)=G1(I,II,JJ)
	END DO

	DO I=1,NP
	y2(i)=0
	Y2(I)=dd+amaxi*Y(I+n2)
	ti(i)=off+ts(i)
	END DO
c	CALL SETUSV('LW',1500)
	CALL SETUSV('LW',1000)
	CALL CURVE(TI,Y2,np-50)
200			end do
			end do
						end do
							end do

1000	call frame
	call clsgks
	stop
	end 

