C Program zoep-sv.f
C Author: J. Pujol
C Solid-solid inteface; incident SV wave.
C Incindence medium: 1
C Transmission medium: 2
C Parameter file: zoep.in

C The subroutine used to solve the 4x4 system of equations was part of a
C commercial package and has been replaced here by a much simpler one
C (see subroutine equation). The new subroutine seems to perform well, 
C but phases of pi may become -pi or viceversa (which is not a problem).

	common amp(4,1000),phase(4,1000),angle(1000),mm(4)
	complex a(4,5),b(4),x(4),y(4,1000)
	COMMON /plot/ nn,XL,XR,YB,YT,FC,px,ppy,delinv,RADINV,fcritpt,
     1fcritpr,fcritst,nfcritpt,nfcritpr,nfcritst,
     1alpha1,beta1,rho1,alpha2,beta2,rho2

	open(4,name='zoep.in',status='old')
	deltaf=.25	!1
	nn=90/deltaf

	read(4,*)alpha1,beta1,rho1,alpha2,beta2,rho2
c	print*,alpha1,beta1,rho1,alpha2,beta2,rho2

	pi=4*atan(1.)
	fac=180/pi	
	radinv=fac
	pi2=radinv*8*atan(1.)
	deltafrad=deltaf/fac

	be1al1=beta1/alpha1
	be2al2=beta2/alpha2
	be2be1=beta2/beta1
	al1be1=alpha1/beta1
	al2be1=alpha2/beta1
	be1al2=1./al2be1
	be2be1=beta2/beta1
	be1be2=1./be2be1
	rho21=rho2/rho1

	fcritpr=asin(be1al1)		!critical angle for reflected P; always
	nfcritpr=fac*fcritpr/deltaf			!exists
c	print*,' al1be1, nfcritpr, fcritpr:',al1be1,nfcritpr,fcritpr*fac

	if(al2be1.gt.1)then		!critical angle for transmitted P
	fcritpt=asin(be1al2)
	nfcritpt=fac*fcritpt/deltaf
	else				!no critical angle for transmitted P
	nfcritpt=nn
	end if
c	print*,' al2be1, nfcritpt, fcritpt:',al2be1,nfcritpt,fcritpt*fac

	if(be2be1.gt.1)then		!critical angle for transmitted S
	fcritst=asin(be1be2)
	nfcritst=fac*fcritst/deltaf
	else				!no critical angle for transmitted S
	nfcritst=nn
	end if
c	print*,' be2be1, nfcritst, fcritst:',be2be1,nfcritst,fcritst*fac

	mm(1)=nfcritpr
	mm(2)=nn
	mm(3)=nfcritpt
	mm(4)=nfcritst
		do i=1,nn
C The last row of column 1, column 2 and vector b are always real 
	f=i*deltafrad
	sin2f=sin(2*f)
	cos2f=cos(2*f)
	b(1)=cmplx(sin(f),0.)
	b(2)=cmplx(cos(f),0.)
	b(3)=cmplx(cos2f,0.)
	b(4)=cmplx(sin2f,0.)
	a(1,2)=cmplx(-sin(f),0.)
	a(2,2)=cmplx(cos(f),0.)
	a(3,2)=cmplx(-cos2f,0.)
	a(4,2)=cmplx(sin2f,0.)
	a(4,1)=cmplx(al1be1*cos2f,0.)

C The first column is real for angles less than fcritpr
	if(i.le.nfcritpr)then
	e=asin(al1be1*sin(f))
	sin2e=sin(2*e)
	a(1,1)=cmplx(-cos(e),0.)
	a(2,1)=cmplx(-sin(e),0.)
	a(3,1)=cmplx(be1al1*sin2e,0.)
	else
	sine=al1be1*sin(f)
	cose=-sqrt(sine**2-1)	!IMAG
	sin2e=2*sine*cose	!IMAG
	a(1,1)=cmplx(0.,-cose)
	a(2,1)=cmplx(-sine,0.)
	a(3,1)=cmplx(0.,be1al1*sin2e)
		sin2e=0			!NOTE!!!
	end if

C The first three rows of column 3 depend on e' only, and are real for
C angles e less than, or equal to, the corresponding critical angle 
	if(i.le.nfcritpt)then
	epr=asin(sin(f)*al2be1)	!e'
	sin2epr=sin(2*epr)
	a(1,3)=cmplx(-cos(epr),0.)
	a(2,3)=cmplx(sin(epr),0.)
	a(3,3)=cmplx(rho21*be2al2*be2be1*sin2epr,0.)
	else
	sinepr=sin(f)*al2be1		!sin(e')
	cosepr=-sqrt(sinepr**2-1)	!cos(e')/i	IMAG
	sin2epr=2*sinepr*cosepr		!sin(2*e')/i	IMAG
	a(1,3)=cmplx(0.,-cosepr)
	a(2,3)=cmplx(sinepr,0.)
	a(3,3)=cmplx(0.,rho21*be2al2*be2be1*sin2epr)
		sin2epr=0		!NOTE!!!
	end if

C The last row of column 3 and column 4 depend on f' only, and are real for
C angles e less than, or equal to, the corresponding critical angle (always
C larger than the previous one). 

	if(i.le.nfcritst)then
	fpr=asin(sin(f)*be2be1)	!f'
	sin2fpr=sin(2*fpr)
	a(4,3)=cmplx(-rho21*al2be1*cos(2*fpr),0.)
	a(1,4)=cmplx(sin(fpr),0.)
	a(2,4)=cmplx(cos(fpr),0.)
	a(3,4)=cmplx(rho21*be2be1*cos(2*fpr),0.)
	a(4,4)=cmplx(rho21*be2be1*sin2fpr,0.)
	else
	sinfpr=sin(f)*be2be1		!sin(f')
	cosfpr=-sqrt(sinfpr**2-1)	!cos(f')/i	IMAG
	sin2fpr=2*sinfpr*cosfpr		!sin(2*f')/i=2*sin(f')*cos(f')/i  IMAG
	cos2fpr=-(cosfpr**2+sinfpr**2)	!cos(2*f')=cos(f')**2-sin(f')**2
	a(4,3)=cmplx(-rho21*al2be1*cos2fpr,0.)
	a(1,4)=cmplx(sinfpr,0.)
	a(2,4)=cmplx(0.,cosfpr)
	a(3,4)=cmplx(rho21*be2be1*cos2fpr,0.)
	a(4,4)=cmplx(0.,rho21*be2be1*sin2fpr)
		sin2fpr=0		!NOTE!!!
	end if

	do ii=1,4
	a(ii,5)=b(ii)
	end do
	call equation(a,x,4)

C Energy equation:
	if(i.eq.nn)go to 40	!incident angle= 90
	en=cabs(x(2))**2+(sin2e*cabs(x(1))**2+rho21*
     1(sin2epr*cabs(x(3))**2+sin2fpr*cabs(x(4))**2))/sin2f

	if(abs(1-en).gt.1.e-5)print*,' Energy equation:',i,en

	angle(i)=i*deltaf
	do j=1,4
	amp(j,i)=cabs(x(j))
	phase2=radinv*atan2(aimag(x(j)),real(x(j)))
	phase(j,i)=phase2
	y(j,i)=x(j)
	end do
40		end do

	call pplot
	stop
	end
c------------------------------------------------------------------------------
	subroutine lwidth(LINEWIDTH)
	CALL SETUSV('LW',LINEWIDTH)
	return
	end
C-------------------------------------------------------------------------------
	subroutine pplot
	dimension x(1000),y(1000)
	common amp(4,1000),phase(4,1000),angle(1000),mm(4)
	COMMON /plot/ nn,XL,XR,YB,YT,FC,px,ppy,delinv,RADINV,fcritpt,
     1fcritpr,fcritst,nfcritpt,nfcritpr,nfcritst,
     1alpha1,beta1,rho1,alpha2,beta2,rho2
	CHARACTER*9 char*60,char2(4)

        CALL OPNGKS   
 	CALL PLCHHQ(.5,.965,'Output of zoep-sv.f',10.,0,0.)    
	CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET
	CALL PLCHHQ(.4,.925,'Solid-solid interface',10.,0,0.)
        CALL PLCHHQ(.4,.9,'Incident SV waves',10.,0,0.)
	if(alpha1.gt.alpha2)then
c        CALL PLCHHQ(.4,.1,'Fig. 6-9',10.,0,0.)
	else
c        CALL PLCHHQ(.4,.1,'Fig. 6-10',10.,0,0.)
	end if

	char='                                                            '
	char(1:27)=':0701:, :0702:, :0721::333:'
	write(char(29:32),'(f4.2)')alpha1
	write(char(35:38),'(f4.2)')beta1
	write(char(41:44),'(f4.2)')rho1
	char(33:33)=','
	char(39:39)=','

	CALL PLCHHQ(.4,.825,char(1:44),10.,0,0.)
	char='                                                            '
	char(1:43)=':0701::0743:,:0702::0743:,:0721::0743::333:'
	write(char(45:48),'(f4.2)')alpha2
	write(char(51:54),'(f4.2)')beta2
	write(char(57:60),'(f4.2)')rho2
	char(49:49)=','
	char(55:55)=','

	CALL PLCHHQ(.4,.85,char,10.,0,0.)

	YB=0
	YT=1.5
	xl=0
	xr=90
	OFFX=21
        OFFY=-.4
        XLL=XL-OFFX
        XRR=XR+OFFX
        YBB=YB+OFFY
        YTT=YT-OFFY/2

        PX=.5*(XL+XR)
        PPY=.5*(YB+YT)
        XLOFF=-.2
        PY=YB+xloff

c Scale the vertical axis for phases
        DELY=380/(YT-YB)
        DELINV=1./DELY
	do j=1,4
	do i=1,nn
	phase(j,i)=delinv*(phase(j,i)+190)
	end do
	end do
	
	char2(1)='A:B1:1 /B'
	char2(2)='B:B1:1 /B'
	char2(3)='A:743:/B '
	char2(4)='B:743:/B  '
			do j=1,4
	if(j.eq.1.or.j.eq.2)then
	WB=.45
	WT=.8
	WL=.1+(j-1)*.3
	WR=.4+(j-1)*.3
	else
	WB=.1
	WT=.45
	WL=.1+(j-3)*.3
	WR=.4+(j-3)*.3
	end if

	CALL SET(WL,WR,WB,WT,XLL,XRR,YBB,YTT,1)
	call axes(j)

	CALL PLCHHQ(60.,1.4,char2(j),10.,0.,-1.)

	CALL SETUSV('LW',1500)

	call line(xl,yb,xr,yb)
	call line(xl,yt,xr,yt)
	call line(xl,yb,xl,yt)
	call line(xr,yb,xr,yt)
	do i=1,10
	xx=(i-1)*10
	call line(xx,yb,xx,yb+.02)
	end do
	do i=1,16
	yy=(i-1)*.1
	call line(xl,yy,xl+1.,yy)
	end do

	amax=0	
	do i=1,nn
	x(i)=angle(i)
	y(i)=amp(j,i)
	if(y(i).gt.amax)amax=y(i)
	if(y(i).gt.yt)y(i)=yt
	end do

	if(amax.gt.yt)then
	write(char(1:4),'(f4.1)')amax
	CALL PLCHHQ(60.,1.25,char(1:4),10.,0,-1.)
	end if

	ISOL=IOR(ISHIFT(32767,1),1)
        CALL DASHDB(ISOL)
	call curve(x,y,nn-1)
        CALL SETUSV('LW',2500)
	call curve(x,y,mm(j)-1)

	CALL SETUSV('LW',1000)

	do i=1,nn
	y(i)=phase(j,i)
	end do
        CALL DASHDC('$''',5,5)
	CALL SETUSV('LW',2000)
	call curved(x,y,nn-1)
			end do

100	call frame
	call clsgks
	stop
	end 
C-------------------------------------------------------------------------------
	subroutine axes(j)
	dimension cchar(5),ind(5)
	COMMON /plot/ nn,XL,XR,YB,YT,FC,px,ppy,delinv,RADINV,fcritpt,
     1fcritpr,fcritst,nfcritpt,nfcritpr,nfcritst,
     1alpha1,beta1,rho1,alpha2,beta2,rho2
	CHARACTER*24 CNUM,cchar

	ind(1)=12
	cchar(1)=':0246::0720:'		!-pi
	ind(2)=24
	cchar(2)=':0246::0720::0517::0635:'	!-pi/2
	ind(3)=6
	cchar(3)=':0633:'			!0
	ind(4)=18
	cchar(4)=':0720::0517::0635:'	!pi/2
	ind(5)=6
	cchar(5)=':0720:'			!pi

	CALL SETUSV('LW',1000)
	CALL LINE(XL,YB,XR,YB)
	CALL LINE(XL,YT,XR,YT)
	CALL LINE(XL,YB,XL,YT)
	CALL LINE(XR,YB,XR,YT)
	if(j.eq.1)then
	CALL PLCHHQ(PX,yb-.25,'Incidence angle',10.,0.,0.)
	CALL PLCHHQ(XL-19,PPY,'Absolute value',10.,90.,0.)
	end if
	if(nfcritpr.lt.nn.and.nfcritpr.gt.0)
     1CALL PLCHHQ(radinv*fcritpr,yb-.05,':734:',12.,0.,0.)	!arrow
	if(nfcritpt.lt.nn.and.nfcritpt.gt.0)
     1CALL PLCHHQ(radinv*fcritpt,yb-.05,':734:',12.,0.,0.)	!arrow
	if(nfcritst.lt.nn.and.nfcritst.gt.0)
     1CALL PLCHHQ(radinv*fcritst,yb-.05,':734:',12.,0.,0.)	!arrow

	do i=1,4
	iang=(i-1)*30
	write(cnum(1:2),'(i2)')iang
	ang=iang
c	call line(ang,yb,ang,yb+.1)
	CALL PLCHHQ(ang,yb-.1,CNUM(1:2),10.,0,0)	
	end do	

	do i=1,10
	amp=(i-1)*.5
	write(cnum(1:3),'(f3.1)')amp
c	call line(xl,amp,xl-2,amp)
	CALL PLCHHQ(xl-8,amp,CNUM(1:3),10.,0.,0)	
	end do	

	if(j.eq.2)CALL PLCHHQ(XR+13,PPY,'Phase',10.,90.,0.)
	DO I=1,5
	IANG=-180+(I-1)*90	
	ANG=DELINV*(iang+190)
	CNUM(1:ind(i))=cchar(i)
	CALL PLCHHQ(XR+2,ANG,CNUM(1:ind(i)),10.,0,-1.)
	CALL LINE(XR,ANG,XR-2,ANG)
	END DO

	return
	end

	SUBROUTINE EQUATION(A,X,N)
	DIMENSION A(4,5),X(4) 
	complex a,x,f

	NP=N+1
	DO 9 I=1,N
	IP=I+1
	DO 9 J=1,N
	IF(I-J)4,9,4
4	F=-A(J,I)/A(I,I)
	DO 3 K=IP,NP
3	A(J,K)=A(J,K)+F*A(I,K)
9	CONTINUE
	DO 10 I=1,N
	X(I)=A(I,N+1)/A(I,I)
10	CONTINUE
	RETURN
	END
