C Program layer-sh-ray.f
C Author: J. Pujol	
C SH waves in a layer over a half-space. Ray theory approach; normal incidence.
C Case of a low velocity layer (rho and beta for the layer and the half space are
C rho1, b1 and rho2, b2).
C The reflection coefficient is ~ 0.914 (Haskell 1960 example). 
C The function f(t) in Fig. 8.9 is in array yy. The peak value of (8.8.13) is
C obtained when nn = 27. This gives an amplitude ratio of 22.2, equal to the 
C theoretical value.
C To compile and link:
C f77 -c fftsub.f
C ncargf77 -w  layer-sh-ray.f fftsub.o -o  layer-sh-ray

	dimension xx(1000),yy(1000),x(16384),y(16384)
	COMPLEX COM(16384)
	common dx,y0(16384),igo,igo2,delt
	common /splot/ wl,wr,wb,wt,xxl,xxr,jgo2
	character*2 char

C INPUT PARAMETERS
	igo2=0
	b1=0.25	!km/s
	rho1=1.7
	b2=3.5	!km/s
	rho2=2.7

        CALL OPNGKS 
	CALL PLCHHQ(.75,.995,'Output of layer-sh-ray.f',10.,0,0.)        
	call dashdc('$''''''$',2,5)
        print*,' Enter 1 for time domain; 2 for frequency domain:'
	read(5,*)jgo0
		if(jgo0.eq.1)then
c	call plchhq(.5,.45,'Fig. 8-9',12.,0,0)	
		else
        print*,' Enter 1 for spectral amplitudes; 2 for ratios:'
	read(5,*)jgo2
	if(jgo2.eq.1)then
c	call plchhq(.45,.4,'Fig. 8-10',12.,0,0)
	else
c	call plchhq(.45,.4,'Fig. 8-11',12.,0,0)
	end if
		end if

	transc=2*rho2*b2/(rho1*b1+rho2*b2)
	refc=(rho1*b1-rho2*b2)/(rho1*b1+rho2*b2)
	print*,' transmission and reflection coefficients and R:',
     1transc,refc,(rho2*b2)/(rho1*b1)
	igo=0
				np=30	!46 2 Apr.
				np=46
	dx=.01
	delt=dx
	nft=8192
	dx2=0.01

1	do i=1,16384
	y(i)=0
	x(i)=0
	end do

        CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET
	YB=-6
	YT=6
	YBb=-4
	YTt=4
	xl=-2
	xr=20.5
	WB=.5
	WT=1
	WL=.1
	WR=.8
        CALL PCSETI('CD',0)    !COMPLEX CHARACTER SET
	CALL SET(WL,WR,WB,WT,XL,XR,YB,YT,1)  

		if(jgo0.eq.2)go to 5
C time axis
	call line(0,0,xr,0)
	call line(0,ybb,xr,ybb)
	call line(0,ybb,0,ytt)
	do i=1,50
	xli=(i-1)*100*dx2
	call line(xli,0,xli,.1)
	call line(xli,ybb,xli,ybb-.1)
	end do

	call plchhq(.5*xr,ybb-1.,'Time (s)',10.,0,0)
	do i=1,5
	xli=(i-1)*500*dx2
	call line(xli,0,xli,.15)
	call line(xli,ybb,xli,ybb-.15)
	if(xli.lt.10)then
	write(char(1:1),'(i1)')int(xli)
	call plchhq(xli,ybb-.5,char(1:1),10.,0,0)
	else
	write(char(1:2),'(i2)')int(xli)
	call plchhq(xli,ybb-.5,char(1:2),10.,0,0)
	end if
	end do

c y axis
	do i=1,10
	yli=ybb+(i-1)
	if(yli.gt.ytt)go to 5
	call line(0,yli,-.2,yli)
	if(yli.lt.0)then
	write(char(1:2),'(i2)')int(yli)
	call plchhq(-.3,yli,char(1:2),10.,0,1.)
	else
	write(char(1:1),'(i1)')int(yli)
	call plchhq(-.3,yli,char(1:1),10.,0,1.)
	end if
	end do

5	avg=0
	pi=4*atan(1.)
	pi2=2*pi
c	per=(2./3)*(np-1)*dx
	per=(np-1)*dx
	avg=0
	do i=1,np
	xx(i)=(i-1)*dx
	yy(i)=sin(pi2*xx(i)/per)
	avg=avg+yy(i)
	end do

	avg=avg/np
	do i=1,np
	yy(i)=yy(i)-avg
	end do

		if(igo.eq.0)then
	do i=1,nft
	com(i)=cmplx(0.,0.)
	end do
	do i=1,np
	com(i)=cmplx(yy(i),0.)
	end do
	call setusv('LW',4000)
				igo=jgo2
		if(jgo0.eq.2)call FFTsub(COM,nft,0)

	do i=1,nft/2
	y0(i)=delt*sqrt(real(com(i))**2+aimag(com(i))**2) !approximate the
	end do				!true F.T. by multiplying with delt
		end if

	igo=jgo2

	call setusv('LW',4000)
		if(jgo0.eq.1)call curve(xx,yy,np)
	do l=1,np
c	write(1,*)l,xx(l),yy(l)
	end do
	call setusv('LW',1000)

	do i=1,np
	x(i)=xx(i)
	y(i)=yy(i)
	end do

	print*,' Enter nn (equal to 27 for this example):'
	read(5,*)nn
	if(nn.eq.0)go to 999
		do k=2,100
		np1=1+nn*(k-1)
		np2=np1+np-1

	ii=0
	do i=np1,np2
	ii=ii+1
	x(i)=(i-1)*dx
	y(i)=y(i)+((refc)**(k-1))*yy(ii)
	end do
		end do
	do i=1,np2
	y(i)=transc*y(i)
	end do

		if(jgo0.eq.1)then
		call curve(x,y,np2)
c	write(1,*)' -----------------------'
	do l=1,100
c	write(1,*)l,x(l),y(l)
	end do
		go to 999
		end if

	do i=1,nft
	com(i)=cmplx(0.,0.)
	end do
	do i=1,np2
	com(i)=cmplx(y(i),0.)
	end do
	igo2=1
	call FFTsub(COM,nft,1)

			if(nn.eq.0)go to 999
c			go to 1
999	call frame
	call clsgks
	stop
	end 
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine fftsub(COM,nft,idash)
	dimension x(16384),y(16384),per(16384),yp(16384)
	complex com(16384)
	common dx,y0(16384),igo,igo2,delt
        common /splot/ wl,wr,wb,wt,xxl,xxr,jgo2

	do i=1,nft
	x(i)=real(com(i))
	y(i)=aimag(com(i))
	end do
	call fft(x,y,nft,nft,nft,-1)
	do i=1,nft
	com(i)=cmplx(x(i),y(i))
	end do

	yb=0
	yt=25
	if(igo.eq.1)yt=5
	XL=0 
	XR=10
	wb=.5
	wt=1
	wl=.1
	wr=.8
c	call set(wl,wr,wb,wt,xl,xr,yb,yt,1)  

	amax=0
	do i=1,nft/2
	y(i)=delt*sqrt(real(com(i))**2+aimag(com(i))**2) !approximate the
	absa=y(i)                        !true F.T. by multiplying with delt
	if(amax.lt.absa)then
	amax=absa
	imax=i
	end if
	end do

	delf=1/(nft*dx)
c	fac=1./amax

	fac=1
	do i=1,nft/2
	y(i)=fac*y(i)
	x(i)=(i-1)*delf
	end do

	if(igo.eq.2.and.igo2.eq.1)then
	do i=2,nft/2
	y(i)=y(i)/y0(i)
c	if(i.le.nft/2)write(7,'(f8.3,f8.2)')1./x(i),y(i)
	end do
	y(1)=y(2)
	end if

	maxft=xr/delf
	xmax=x(imax)

	xl=.05
	xr=6 
	xr=10
       	xxl=xl
        xxr=xr

	f1=1./xl
	f2=1./xr
	nf1=2+f1/delf
	nf2=2+f2/delf

	ii=0
	do i=nf2,nf1
	ii=ii+1
	per(ii)=ALOG10(1./x(i))
	yp(ii)=y(i)
	end do
	nper=ii

        xl=alog10(xl)
        xr=alog10(xr)
	call plot(xl,xr,yb,yt,xll,ybb,xrr,hx)
	if(idash.eq.0)then
	if(jgo2.eq.1)then
	call setusv('LW',4000)
	call curve(per,yp,nper)
	do k=1,nper
	yp(k)=10*yp(k)
	end do
	call setusv('LW',1000)
	call curved(per,yp,nper)	!plotting purposes
	do k=1,nper
	yp(k)=.1*yp(k)
	end do
	end if
	else
	call curve(per,yp,nper)
	end if

	if(jgo2.eq.1)then
	xmax=alog10(1./xmax)
	call line(xmax,0,xmax,.04*yt)
	call plchhq(xmax,.03*yt,':1347:',8.,0,0)
	elseif(jgo2.eq.2.and.idash.eq.1)then
	xmaxt=alog10(1./xmax)
	call plchhq(xmaxt,.03*yt,':1347:',8.,0,0)
	xmax1=1./xmax
	do k=1,5
	xmax=xmax1/(2*(k-1)+1)
	xmax=alog10(xmax)
	call line(xmax,0,xmax,.04*yt)
	end do
	end if

	YB=-6
	YT=6
	YBb=-4
	YTt=4
	xl=-2
	xr=20.25
	WB=.5
	WT=1
	call set(wl,wr,wb,wt,xl,xr,yb,yt,1)  
	return
	end
c-----------------------------------------------------------------------------------------
	subroutine plot(xl,xr,yb,yt,xll,ybb,xrr,hx)
	common /splot/ wl,wr,wb,wt,xxl,xxr,jgo2
	common /model/h,betal,rhol,beta,rho,f,rad,qfac
	character*3 char,char2*41,char3*54

	call setusv('LW',1500)
	char2='                                         '
	char2(1:30)=':0702::0743:,:0721::0743::333:'
	write(char2(32:35),'(f4.2)')betal
	write(char2(38:41),'(f4.2)')rhol
	char2(36:36)=','
c	CALL PLCHHQ(.4,.85,char2,10.,0,0.)
	char2='                                         '
	char2(1:19)=':0702:, :0721::333:'
	write(char2(21:24),'(f4.2)')beta
	write(char2(27:30),'(f4.2)')rho
	char2(25:25)=','
c	CALL PLCHHQ(.4,.825,char2(1:30),10.,0,0.)
	char3='                                         '
	char3(1:6)='h:333:'
	write(char3(8:11),'(f4.1)')h
	char3(13:14)='km'
	char3(20:25)='f:333:'
	iang=f/rad
	write(char3(27:29),'(i3)')iang
	char3(30:38)=':S1::632:'

	char3(45:50)='Q:333:'
	iqfac=qfac
	write(char3(52:53),'(i2)')iqfac

c	CALL PLCHHQ(.4,.8,char3,10.,0,0.)

	wb=.5
	wt=1
	wl=.1
	wr=.8
         
	dx=xr-xl
	dy=yt-yb
	xll=xl-.11*dx
	xrr=xr+.11*dx
	ybb=yb-.15*dy
	ytt=yt+.05*dy
	CALL SET(WL,WR,WB,WT,XLl,XRr,YBb,YTt,1)  

	hx=.02*dx
	hy=.025*dy
	hx2=hx/2
	hy2=hy/2

	call plchhq(.5*(xll+xrr),yb-4.2*hy,'Period (s)',10.,0,0)

	dely=.5
	if(dy.gt.10)dely=5
	if(dy.gt.100)dely=50
                igo=0
                if(igo.eq.0)go to 15
	delx=.5
	if(dx.gt.10)delx=5
	if(dx.gt.100)delx=50

C Period tick marks (not used)
	ii=0
	do i=1,40
	xp=(i-1)*delx
	if(xp.lt.xl)go to 10
	if(xp.gt.xr)go to 20
	ii=ii+1
	call line(xp,yb,xp,yb-hy2)
	
	if(ii.eq.2*(ii/2))then
	call line(xp,yb,xp,yb-hy)
	call cchar(xp,char,ic)
	call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0) 
	end if
10	end do

C Period tick marks (logarithmic)
15	if(xxl.lt..1)then
	xp=xxl
	write(char(1:4),'(f4.2)')xp
        xp=alog10(xp)
	call plchhq(xp,yb-1.8*hy,char(1:4),10.,0,0)
        call line(xp,yb,xp,yb-.9*hy)
	end if


	do i=1,9
        xp=i*.1
        if(xp.lt.xxl)go to 12
        if(xp.gt.xxr)go to 13 
	write(char(1:3),'(f3.1)')xp
        xp=alog10(xp)
        if(i.eq.2*(i/2).and.i.lt.8)
     1call plchhq(xp,yb-1.8*hy,char(1:3),10.,0,0)
        call line(xp,yb,xp,yb-.9*hy)
12      end do
      
13	do i=1,9
        xp=i
        if(xp.lt.xxl)go to 16 
        if(xp.gt.xxr)go to 17 
        call cchar(xp,char,ic)
        xp=alog10(xp)
        if(i.ne.2*(i/2).and.i.lt.8)
     1call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0)
        call line(xp,yb,xp,yb-hy)
16      end do
17      ii=0  
        do i=10,100,10
        ii=ii+1
        xp=i
        if(xp.gt.xxr)go to 20
        call cchar(xp,char,ic)
        xp=alog10(xp)
        if(ii.ne.2*(ii/2))call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0)
        call line(xp,yb,xp,yb-hy)
        end do

C Amplitude tick marks
20	do i=1,40
	yp=(i-1)*dely
	if(yp.lt.yb)go to 30
	if(yp.gt.yt)go to 40
	if(dely.eq.5)call line(xl,yp,xl-.8*hx2,yp)	!7 Mar01
30	end do

40	do i=1,40
	yp=2*(i-1)*dely
	if(dely.eq.5)yp=(i-1)*dely	!7 Mar01
	if(yp.lt.yb)go to 35
	if(yp.gt.yt)go to 45
	call line(xl,yp,xl-.8*hx,yp)
	call cchar(yp,char,ic)
	call plchhq(xl-1.*hx,yp,char(1:ic),10.,0,1.) 
35	end do

45	if(jgo2.eq.2)then
	call plchhq(xl-4.5*hx,.5*(yb+yt),'Spectral ratio',10.,90.,0) 
	else
	call plchhq(xl-4.4*hx,.5*(yb+yt),'Spectral amplitude',10.,90.,0) 
	end if

	call line(xl,yb,xr,yb)
	call line(xl,yb,xl,yt)
c	call line(xr,yb,xr,yt)
	return
	end 
c------------------------------------------------------------------------------
	subroutine cchar(xp,char,ic)
	character*3 char
	ip=xp
	char='   '
	if(0.le.ip.and.ip.le.9)then
	ic=1
	write(char(1:1),'(i1)')ip
	elseif(10.le.ip.and.ip.le.99)then
	ic=2
	write(char(1:2),'(i2)')ip
	elseif(100.le.ip.and.ip.le.999)then
	ic=3
	write(char(1:3),'(i3)')ip
	end if

	return
	end
