C Program sh-layer-q.f 
C Author: J. Pujol	
C Solid layer over solid half-space. Incident SH wave.

C betal, beta: S-wave velocity in the layer and half-space
C rhol,rho: density in the layer and half-space
C h: layer thickness
C rmu: ratio between mu for layer and mu for half-space
C f: incidence angle at the bottom of layer
C u0: surface displacement
C wk: wave number k = omega/c
C To account for attenuation in the layer the velocity betal is multiplied by 
C (1 + i/2Q)

C Input file: sh-layer.model

	dimension amp(10000),phase(10000),per(10000),perm(100)
	complex u0,ci,theta,eta,etal,etalc,costh,sinth,cossq,sinsq
     1,rmu,rmusq,cone,cfac,cbetal,betalsq,wk
        common /splot/ wl,wr,wb,wt,xxl,xxr,igo1,iang
	common /model/h,betal,rhol,beta,rho,f,rad,qfac
	call dashdc('$''''''$',2,5)

	cone=cmplx(1.,0.)
	ci=cmplx(0.,1.)
	pi=4*atan(1.)
	twopi=2*pi
	rad=pi/180
	print*,' Enter angle f, min. and max. period:'
	read(5,*)f,xl1,xr1
	iang=f

	print*,' Enter H, beta and rho of layer and half-space, and Q of 
     1layer:'
	open(1,name='sh-layer.model',status='old')
	read(1,*)h,betal,rhol,beta,rho,qfac

	f=rad*f
	c=beta/sin(f)
	csq=c*c
	betasq=beta**2

	fac=csq/betasq
	eta=cmplx(sqrt(fac-1),0.)	!eta for half-space

	igo1=0
1	if(igo1.eq.0)then
	q=1.e+6
	elseif(igo1.eq.1)then
	q=qfac
	elseif(igo1.eq.2)then
	go to 999
	end if

        xxl=xl1
        xxr=xr1

	cbetal=betal*(1+ci/(2*q))
 
	betalsq=cbetal**2
	cfac=csq/betalsq
	etal=csqrt(cone*(cfac-1))

	rmu=(rhol*betalsq*etal)/(rho*betasq*eta)
	rmusq=rmu*rmu

11	nper=10000	!5000
	dper=(xr1-xl1)/nper

		do i=1,nper
	per(i)=xl1+dper*i

	freq=1./per(i)
	omega=twopi*freq
	wk=omega/c
	theta=h*wk*etal
	costh=ccos(theta)
	sinth=csin(theta)
	cossq=costh**2
	sinsq=sinth**2
	u0=2/(costh+rmu*ci*sinth)
	amp(i)=cabs(u0)

	phase(i)=atan2(aimag(u0),real(u0))
		end do

	if(igo1.eq.0)then
	amax=0
	pmax=xr
	pmin=0
	do  i=1,nper
	if(amp(i).gt.amax)amax=amp(i)
	end do
	end if

	if(amax.lt.2)amax=2

	yb=0
	yt=amax
	yt=1.01*amax	
        xl=alog10(xl1)
        xr=alog10(xr1)

	do i=1,nper
	per(i)=ALOG10(per(i))
	end do

	call plot(xl,xr,yb,yt,xll,ybb,xrr,hx)

	call lwidth(2000)

	if(igo1.eq.0)then
	call curve(per,amp,nper)
	else
	call lwidth(3000)
	call curved(per,amp,nper)
	end if

	yt=20
	ybb=-yt
	CALL SET(WL,WR,WB,WT,XLl,XRr,YBb,YT,1)  
	call lwidth(1000)

	if(igo1.eq.0)call curved(per,phase,nper)

c	if(igo.eq.0)go to 999
C Phase tick marks
	call line (per(nper),pi,per(nper)-hx,pi)
	call plchhq(per(nper)+hx,pi,':1120:',14.,0,0) 	!pi
	call line (per(nper),-pi,per(nper)-hx,-pi)
	call plchhq(per(nper)+2*hx,-pi,':446::1120:',14.,0,0) !-pi
c	call plchhq(per(nper)+1.007*hx,-pi,':0046:',12.,0,0) 
	call line (per(nper),0.,per(nper)-hx,0.)
	call plchhq(per(nper)+hx,0,' 0',7.,0,0) 
	call plchhq(per(nper)+3.5*hx,0.,'Phase',10.,90.,0) 

				igo1=igo1+1
				go to 1
999	call frame
	call clsgks
	stop
	end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
	subroutine plot(xl,xr,yb,yt,xll,ybb,xrr,hx)
        common /splot/ wl,wr,wb,wt,xxl,xxr,igo1,iang
	common /model/h,betal,rhol,beta,rho,f,rad,qfac
	character*3 char,char2*41,char3*54

	if(igo1.eq.1)go to 11

        CALL OPNGKS   
 	CALL PLCHHQ(.5,.965,'Output of sh-layer-q.f',10.,0,0.)    
        CALL PCSETI('CD',1)    !DUPLEX CHARACTER SET
c	CALL PLCHHQ(.4,.925,'sh-layer-q -- Layer over a half-space',10.,0,0.)
	CALL PLCHHQ(.4,.925,'Layer over a half-space',10.,0,0.)
        CALL PLCHHQ(.4,.9,'Incident SH wave',10.,0,0.)
c	CALL PLCHHQ(.4,.1,'Fig. 6-12',10.,0,0.)

	char2='                                         '
	char2(1:30)=':0702::0743:,:0721::0743::333:'
	write(char2(32:35),'(f4.2)')betal
	write(char2(38:41),'(f4.2)')rhol
	char2(36:36)=','

	CALL PLCHHQ(.4,.85,char2,10.,0,0.)
	char2='                                         '
	char2(1:19)=':0702:, :0721::333:'
	write(char2(21:24),'(f4.2)')beta
	write(char2(27:30),'(f4.2)')rho
	char2(25:25)=','

	CALL PLCHHQ(.4,.825,char2(1:30),10.,0,0.)
	char3='                                         '
	char3(1:6)='h:333:'
	write(char3(8:11),'(f4.1)')h
	char3(13:14)='km'
	char3(20:25)='f:333:'
	write(char3(27:29),'(i3)')iang
	char3(30:38)=':S1::632:'

	char3(45:50)='Q:333:'
	iqfac=qfac
	write(char3(52:53),'(i2)')iqfac

	CALL PLCHHQ(.4,.8,char3,10.,0,0.)

11	WB=.2
	WT=.7
	WL=.1
	WR=.7

	dx=xr-xl
	dy=yt-yb
	xll=xl-.11*dx
	xrr=xr+.11*dx
	ybb=yb-.1*dy
	CALL SET(WL,WR,WB,WT,XLl,XRr,YBb,YT,1)  

	hx=.02*dx
	hy=.025*dy
	hx2=hx/2
	hy2=hy/2

	call plchhq(.5*(xll+xrr),yb-3.5*hy,'Period (sec)',10.,0,0)

	dely=.5
	if(dy.gt.10)dely=5
                igo=0
                if(igo.eq.0)go to 15
	delx=.5
	if(dx.gt.10)delx=5

C Period tick marks (not used)
	ii=0
	do i=1,40
	xp=(i-1)*delx
	if(xp.lt.xl)go to 10
	if(xp.gt.xr)go to 20
	ii=ii+1
	call line(xp,yb,xp,yb-hy2)
	
	if(ii.eq.2*(ii/2))then
	call line(xp,yb,xp,yb-hy)
	call cchar(xp,char,ic)
	call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0) 
	end if
10	end do

C Period tick marks (logarithmic)
15	if(xxl.lt..1)then
	xp=xxl
	write(char(1:4),'(f4.2)')xp
        xp=alog10(xp)
	call plchhq(xp,yb-1.8*hy,char(1:4),10.,0,0)
        call line(xp,yb,xp,yb-.9*hy)
	end if

	do i=1,9
        xp=i*.1
        if(xp.lt.xxl)go to 12
        if(xp.gt.xxr)go to 13 
	write(char(1:3),'(f3.1)')xp
        xp=alog10(xp)
        if(i.eq.2*(i/2).and.i.lt.8)
     1call plchhq(xp,yb-1.8*hy,char(1:3),10.,0,0)
        call line(xp,yb,xp,yb-.9*hy)
12      end do
      
13	do i=1,9
        xp=i
        if(xp.lt.xxl)go to 16 
        if(xp.gt.xxr)go to 17 
        call cchar(xp,char,ic)
        xp=alog10(xp)
        if(i.ne.2*(i/2).and.i.lt.8)
     1call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0)
        call line(xp,yb,xp,yb-hy)
16      end do
17      ii=0  
        do i=10,100,10
        ii=ii+1
        xp=i
        if(xp.gt.xxr)go to 20
        call cchar(xp,char,ic)
        xp=alog10(xp)
        if(ii.ne.2*(ii/2))call plchhq(xp,yb-2*hy,char(1:ic),10.,0,0)
        call line(xp,yb,xp,yb-hy)
        end do

C Amplitude tick marks
20	do i=1,40
	yp=(i-1)*dely
	if(yp.lt.yb)go to 30
	if(yp.gt.yt)go to 40
	if(dely.eq.5)call line(xl,yp,xl-.8*hx2,yp)	!7 Mar01
30	end do

40	do i=1,40
	yp=2*(i-1)*dely
	if(dely.eq.5)yp=(i-1)*dely	!7 Mar01
	if(yp.lt.yb)go to 35
	if(yp.gt.yt)go to 45
	call line(xl,yp,xl-hx,yp)
	call cchar(yp,char,ic)
	call plchhq(xl-1.*hx,yp,char(1:ic),10.,0,1.) 
35	end do

45	call plchhq(xl-4.5*hx,.5*(yb+yt),'Spectral amplitude',10.,90.,0) 

	call line(xl,yb,xr,yb)
	call line(xl,yb,xl,yt)
	call line(xr,yb,xr,yt)
	return
	end 
c------------------------------------------------------------------------------
	subroutine lwidth(LINEWIDTH)
	CALL SETUSV('LW',LINEWIDTH)
	return
	end
c------------------------------------------------------------------------------
	subroutine cchar(xp,char,ic)
	character*3 char
	ip=xp
	char='   '
	if(0.le.ip.and.ip.le.9)then
	ic=1
	write(char(1:1),'(i1)')ip
	elseif(10.le.ip.and.ip.le.99)then
	ic=2
	write(char(1:2),'(i2)')ip
	elseif(100.le.ip.and.ip.le.999)then
	ic=3
	write(char(1:3),'(i3)')ip
	end if

	return
	end

